@@ -1,6 +1,7 @@
+/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
 /*
  * Copyright 1993 by OpenVision Technologies, Inc.
- * 
+ *
  * Permission to use, copy, modify, distribute, and sell this software
  * and its documentation for any purpose is hereby granted without fee,
  * provided that the above copyright notice appears in all copies and
@@ -10,7 +11,7 @@
  * without specific, written prior permission. OpenVision makes no
  * representations about the suitability of this software for any
  * purpose.  It is provided "as is" without express or implied warranty.
- * 
+ *
  * OPENVISION DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
  * EVENT SHALL OPENVISION BE LIABLE FOR ANY SPECIAL, INDIRECT OR
@@ -21,7 +22,7 @@
  */
 
 /*
- * $Id: util_buffer.c 11001 1998-10-30 02:56:35Z marc $
+ * $Id: util_buffer.c 23457 2009-12-08 00:04:48Z tlyu $
  */
 
 #include "gssapiP_generic.h"
@@ -31,18 +32,14 @@
    make sure that buffer is consistent (release'able) when this
    function exits, no matter what the exit value */
 
-int g_make_string_buffer(str, buffer)
-     const char *str;
-     gss_buffer_t buffer;
+int g_make_string_buffer(const char *str, gss_buffer_t buffer)
 {
-   buffer->length = strlen(str);
+    buffer->length = strlen(str);
 
-   if ((buffer->value = (void *) xmalloc(buffer->length + 1)) == NULL) {
-      buffer->length = 0;
-      return(0);
-   }
+    if ((buffer->value = strdup(str)) == NULL) {
+        buffer->length = 0;
+        return(0);
+    }
 
-   strcpy(buffer->value, str);
-
-   return(1);
+    return(1);
 }
