@@ -1,6 +1,7 @@
+/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
 /*
  * Copyright 1993 by OpenVision Technologies, Inc.
- * 
+ *
  * Permission to use, copy, modify, distribute, and sell this software
  * and its documentation for any purpose is hereby granted without fee,
  * provided that the above copyright notice appears in all copies and
@@ -10,7 +11,7 @@
  * without specific, written prior permission. OpenVision makes no
  * representations about the suitability of this software for any
  * purpose.  It is provided "as is" without express or implied warranty.
- * 
+ *
  * OPENVISION DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
  * EVENT SHALL OPENVISION BE LIABLE FOR ANY SPECIAL, INDIRECT OR
@@ -24,31 +25,30 @@
 
 OM_uint32
 krb5_gss_release_name(minor_status, input_name)
-     OM_uint32 *minor_status;
-     gss_name_t *input_name;
+    OM_uint32 *minor_status;
+    gss_name_t *input_name;
 {
-   krb5_context context;
-   krb5_error_code code;
+    krb5_context context;
+    krb5_error_code code;
 
-   code = krb5_init_context(&context);
-   if (code) {
-       *minor_status = code;
-       return GSS_S_FAILURE;
-   }
+    code = krb5_gss_init_context(&context);
+    if (code) {
+        *minor_status = code;
+        return GSS_S_FAILURE;
+    }
 
-   if (! kg_validate_name(*input_name)) {
-      *minor_status = (OM_uint32) G_VALIDATE_FAILED;
-      krb5_free_context(context);
-      return(GSS_S_CALL_BAD_STRUCTURE|GSS_S_BAD_NAME);
-   }
+    if (! kg_validate_name(*input_name)) {
+        *minor_status = (OM_uint32) G_VALIDATE_FAILED;
+        krb5_free_context(context);
+        return(GSS_S_CALL_BAD_STRUCTURE|GSS_S_BAD_NAME);
+    }
 
-   (void)kg_delete_name(*input_name);
+    kg_release_name(context, KG_INIT_NAME_INTERN,
+                    (krb5_gss_name_t *)input_name);
+    krb5_free_context(context);
 
-   krb5_free_principal(context, (krb5_principal) *input_name);
-   krb5_free_context(context);
+    *input_name = (gss_name_t) NULL;
 
-   *input_name = (gss_name_t) NULL;
-
-   *minor_status = 0;
-   return(GSS_S_COMPLETE);
+    *minor_status = 0;
+    return(GSS_S_COMPLETE);
 }
