@@ -1,7 +1,8 @@
+/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
 /*
  * lib/gssapi/krb5/rel_oid.c
  *
- * Copyright 1995 by the Massachusetts Institute of Technology.
+ * Copyright 1995, 2007 by the Massachusetts Institute of Technology.
  * All Rights Reserved.
  *
  * Export of this software from the United States of America may
@@ -30,14 +31,14 @@
  */
 #include "gssapiP_krb5.h"
 
-static OM_uint32 krb5_gss_internal_release_oid (OM_uint32 *, /* minor_status */
-						gss_OID * /* oid */
-    );
+OM_uint32 krb5_gss_internal_release_oid (OM_uint32 *, /* minor_status */
+                                         gss_OID * /* oid */
+);
 
 OM_uint32
 krb5_gss_release_oid(minor_status, oid)
-    OM_uint32	*minor_status;
-    gss_OID	*oid;
+    OM_uint32   *minor_status;
+    gss_OID     *oid;
 {
     /*
      * The V2 API says the following!
@@ -49,37 +50,37 @@ krb5_gss_release_oid(minor_status, oid)
      * allocated OID values with OIDs returned by GSS-API.
      */
     if (krb5_gss_internal_release_oid(minor_status, oid) != GSS_S_COMPLETE) {
-	/* Pawn it off on the generic routine */
-	return(generic_gss_release_oid(minor_status, oid));
+        /* Pawn it off on the generic routine */
+        return(generic_gss_release_oid(minor_status, oid));
     }
     else {
-	*oid = GSS_C_NO_OID;
-	*minor_status = 0;
-	return(GSS_S_COMPLETE);
+        *oid = GSS_C_NO_OID;
+        *minor_status = 0;
+        return(GSS_S_COMPLETE);
     }
 }
 
-static OM_uint32
+OM_uint32
 krb5_gss_internal_release_oid(minor_status, oid)
-    OM_uint32	*minor_status;
-    gss_OID	*oid;
+    OM_uint32   *minor_status;
+    gss_OID     *oid;
 {
     /*
      * This function only knows how to release internal OIDs. It will
      * return GSS_S_CONTINUE_NEEDED for any OIDs it does not recognize.
      */
-   
+
+    *minor_status = 0;
     if ((*oid != gss_mech_krb5) &&
-	(*oid != gss_mech_krb5_old) &&
-	(*oid != gss_nt_krb5_name) &&
-	(*oid != gss_nt_krb5_principal)) {
-	/* We don't know about this OID */
-	return(GSS_S_CONTINUE_NEEDED);
+        (*oid != gss_mech_krb5_old) &&
+        (*oid != gss_mech_krb5_wrong) &&
+        (*oid != gss_nt_krb5_name) &&
+        (*oid != gss_nt_krb5_principal)) {
+        /* We don't know about this OID */
+        return(GSS_S_CONTINUE_NEEDED);
     }
     else {
-	*oid = GSS_C_NO_OID;
-	*minor_status = 0;
-	return(GSS_S_COMPLETE);
+        *oid = GSS_C_NO_OID;
+        return(GSS_S_COMPLETE);
     }
 }
-
