@@ -33,6 +33,8 @@
 
 #include<strsafe.h>
 
+#pragma warning(disable: 4204 4221)
+
 typedef struct tag_k5_realm_kdc {
     wchar_t       name[K5_MAXCCH_HOST];
     khm_boolean   admin;        /* admin server? */
@@ -77,6 +79,11 @@ typedef struct tag_k5_config_data {
     khm_boolean   create_config_file; /* create config_file if missing? */
     khm_boolean   inc_realms;   /* include full realm list in new
                                    credentials dialog? */
+    wchar_t       loaded_config_file[MAX_PATH]; /* path to the
+                                                   configuration file
+                                                   that has been
+                                                   loaded into the
+                                                   realm editor. */
 
     /* [libdefaults] */
     khm_boolean   dns_lookup_kdc;
@@ -115,35 +122,6 @@ typedef struct tag_k5_config_data {
 #define K5_CDFLAG_MOD_INC_REALMS     0x00000100
 #define K5_CDFLAG_MOD_REALMS         0x00001000
 
-static const char *const conf_yes[] = {
-    "y", "yes", "true", "t", "1", "on",
-    0,
-};
-
-static const char *const conf_no[] = {
-    "n", "no", "false", "nil", "0", "off",
-    0,
-};
-
-int
-k5_parse_boolean(const char *s)
-{
-    const char *const *p;
-
-    for(p=conf_yes; *p; p++) {
-        if (!_stricmp(*p,s))
-            return 1;
-    }
-
-    for(p=conf_no; *p; p++) {
-        if (!_stricmp(*p,s))
-            return 0;
-    }
-
-    /* Default to "no" */
-    return 0;
-}
-
 void
 k5_init_config_data(k5_config_data * d) {
     ZeroMemory(d, sizeof(*d));
@@ -258,6 +236,7 @@ k5_is_profile_loaded(void) {
     assert(pprofile_add_relation);
     assert(pprofile_update_relation);
     assert(pprofile_flush);
+    assert(pprofile_rename_section);
 #endif
 
     if (!pprofile_init ||
@@ -271,7 +250,8 @@ k5_is_profile_loaded(void) {
         !pprofile_clear_relation ||
         !pprofile_add_relation ||
         !pprofile_update_relation ||
-        !pprofile_flush)
+        !pprofile_flush ||
+        !pprofile_rename_section)
 
         return FALSE;
 
@@ -320,7 +300,12 @@ k5_read_config_data(k5_config_data * d) {
         rv = pprofile_get_string(profile, "libdefaults", "dns_lookup_kdc",
                                  NULL, NULL, &boolv);
         if (!rv && boolv) {
-            d->dns_lookup_kdc = k5_parse_boolean(boolv);
+            khm_boolean b;
+
+            if (!khm_krb5_parse_boolean(boolv, &b))
+                d->dns_lookup_kdc = b;
+            else
+                d->dns_lookup_kdc = FALSE;
             pprofile_release_string(boolv);
         } else
             d->dns_lookup_kdc = FALSE;
@@ -328,7 +313,12 @@ k5_read_config_data(k5_config_data * d) {
         rv = pprofile_get_string(profile, "libdefaults", "dns_lookup_realm",
                                  NULL, NULL, &boolv);
         if (!rv && boolv) {
-            d->dns_lookup_realm = k5_parse_boolean(boolv);
+            khm_boolean b;
+
+            if (!khm_krb5_parse_boolean(boolv, &b))
+                d->dns_lookup_realm = b;
+            else
+                d->dns_lookup_realm = FALSE;
             pprofile_release_string(boolv);
         } else
             d->dns_lookup_realm = FALSE;
@@ -336,7 +326,12 @@ k5_read_config_data(k5_config_data * d) {
         rv = pprofile_get_string(profile, "libdefaults", "dns_fallback",
                                  NULL, NULL, &boolv);
         if (!rv && boolv) {
-            d->dns_fallback = k5_parse_boolean(boolv);
+            khm_boolean b;
+
+            if (!khm_krb5_parse_boolean(boolv, &b))
+                d->dns_fallback = b;
+            else
+                d->dns_fallback = FALSE;
             pprofile_release_string(boolv);
         } else
             d->dns_fallback = FALSE;
@@ -344,7 +339,12 @@ k5_read_config_data(k5_config_data * d) {
         rv = pprofile_get_string(profile, "libdefaults", "noaddresses",
                                  NULL, NULL, &boolv);
         if (!rv && boolv) {
-            d->noaddresses = k5_parse_boolean(boolv);
+            khm_boolean b;
+
+            if (!khm_krb5_parse_boolean(boolv, &b))
+                d->noaddresses = b;
+            else
+                d->noaddresses = TRUE;
             pprofile_release_string(boolv);
         } else
             d->noaddresses = TRUE;
@@ -364,7 +364,7 @@ k5_read_config_data(k5_config_data * d) {
         /* now go through each and fish out the kdcs, admin_server
            and master_kdc. */
         for (s=0; sections[s] && sections[s][0]; s++) {
-            const char * sec_kdcs[] = 
+            const char * sec_kdcs[] =
                 { "realms", sections[s], "kdc", NULL };
 
             const char * sec_admin[] =
@@ -533,7 +533,7 @@ k5_read_config_data(k5_config_data * d) {
     d->flags = 0;
 }
 
-void
+int
 k5_write_config_data(k5_config_data * d) {
     char astr[MAX_PATH * 2];
     char config_file[MAX_PATH];
@@ -541,35 +541,30 @@ k5_write_config_data(k5_config_data * d) {
     const char *filenames[2];
     long rv;
     khm_size s;
+    int applied = FALSE;
 
     if (d->flags == 0)
-        return;
+        return FALSE;
 
     if (!k5_is_profile_loaded())
-        return;
-
-    if (d->flags & K5_CDFLAG_MOD_DEF_REALM) {
-        if (SUCCEEDED(StringCbLength(d->def_realm,
-                                     sizeof(d->def_realm), &s)) &&
-            s > 0) {
-            khm_krb5_set_default_realm(d->def_realm);
-        }
-        d->flags &= ~K5_CDFLAG_MOD_DEF_REALM;
-    }
+        return FALSE;
 
     /* write the MSLSA import setting */
     if (d->flags & K5_CDFLAG_MOD_LSA_IMPORT) {
         khc_write_int32(csp_params, L"MsLsaImport", d->lsa_import);
         d->flags &= ~K5_CDFLAG_MOD_LSA_IMPORT;
+        applied = TRUE;
     }
 
     if (d->flags & K5_CDFLAG_MOD_INC_REALMS) {
         khc_write_int32(csp_params, L"UseFullRealmList", d->inc_realms);
         d->flags &= ~K5_CDFLAG_MOD_INC_REALMS;
+        applied = TRUE;
     }
 
-    if (!(d->flags & 
-          (K5_CDFLAG_MOD_CONF_FILE |
+    if (!(d->flags &
+          (K5_CDFLAG_MOD_DEF_REALM |
+           K5_CDFLAG_MOD_CONF_FILE |
            K5_CDFLAG_MOD_DNS_FALLBACK |
            K5_CDFLAG_MOD_DNS_LOOKUP_RLM |
            K5_CDFLAG_MOD_DNS_LOOKUP_KDC |
@@ -578,7 +573,7 @@ k5_write_config_data(k5_config_data * d) {
            K5_CDFLAG_MOD_REALMS))) {
 
         d->flags = 0;
-        return;
+        return applied;
     }
 
     khm_krb5_get_profile_file(config_file, ARRAYLENGTH(config_file));
@@ -615,6 +610,28 @@ k5_write_config_data(k5_config_data * d) {
         const char * sec_libdefaults[] = { "libdefaults", NULL, NULL };
         khm_size r;
 
+        if (d->flags & K5_CDFLAG_MOD_DEF_REALM) {
+            if (SUCCEEDED(StringCbLength(d->def_realm,
+                                         sizeof(d->def_realm), &s)) &&
+                s > 0) {
+                char defrealm[K5_MAXCCH_REALM];
+
+                UnicodeStrToAnsi(defrealm, sizeof(defrealm),
+                                 d->def_realm);
+
+                khm_krb5_set_default_realm(d->def_realm);
+
+                sec_libdefaults[1] = "default_realm";
+
+                pprofile_clear_relation(profile, sec_libdefaults);
+
+                rv = pprofile_add_relation(profile, sec_libdefaults,
+                                           defrealm);
+                applied = TRUE;
+            }
+            d->flags &= ~K5_CDFLAG_MOD_DEF_REALM;
+        }
+
         if (d->flags & K5_CDFLAG_MOD_DNS_LOOKUP_KDC) {
 
             sec_libdefaults[1] = "dns_lookup_kdc";
@@ -623,12 +640,12 @@ k5_write_config_data(k5_config_data * d) {
 
             rv = pprofile_add_relation(profile, sec_libdefaults,
                                        (d->dns_lookup_kdc)?
-                                       conf_yes[0]:
-                                       conf_no[0]);
+                                       KRB5_CONF_YES:
+                                       KRB5_CONF_NO);
             d->flags &= ~K5_CDFLAG_MOD_DNS_LOOKUP_KDC;
+            applied = TRUE;
         }
 
-
         if (d->flags & K5_CDFLAG_MOD_DNS_LOOKUP_RLM) {
 
             sec_libdefaults[1] = "dns_lookup_realm";
@@ -637,10 +654,11 @@ k5_write_config_data(k5_config_data * d) {
 
             rv = pprofile_add_relation(profile, sec_libdefaults,
                                        (d->dns_lookup_realm)?
-                                       conf_yes[0]:
-                                       conf_no[0]);
+                                       KRB5_CONF_YES:
+                                       KRB5_CONF_NO);
 
             d->flags &= ~K5_CDFLAG_MOD_DNS_LOOKUP_RLM;
+            applied = TRUE;
         }
 
         if (d->flags & K5_CDFLAG_MOD_DNS_FALLBACK) {
@@ -651,10 +669,11 @@ k5_write_config_data(k5_config_data * d) {
 
             rv = pprofile_add_relation(profile, sec_libdefaults,
                                        (d->dns_fallback)?
-                                       conf_yes[0]:
-                                       conf_no[0]);
+                                       KRB5_CONF_YES:
+                                       KRB5_CONF_NO);
 
             d->flags &= ~K5_CDFLAG_MOD_DNS_FALLBACK;
+            applied = TRUE;
         }
 
         if (d->flags & K5_CDFLAG_MOD_NOADDRESSES) {
@@ -665,10 +684,11 @@ k5_write_config_data(k5_config_data * d) {
 
             rv = pprofile_add_relation(profile, sec_libdefaults,
                                        (d->noaddresses)?
-                                       conf_yes[0]:
-                                       conf_no[0]);
+                                       KRB5_CONF_YES:
+                                       KRB5_CONF_NO);
 
             d->flags &= ~K5_CDFLAG_MOD_NOADDRESSES;
+            applied = TRUE;
         }
 
         /* now we look at the [realms] section */
@@ -677,7 +697,7 @@ k5_write_config_data(k5_config_data * d) {
             char realm[K5_MAXCCH_REALM];
             char host[K5_MAXCCH_HOST];
 
-            const char * sec_kdcs[] = 
+            const char * sec_kdcs[] =
                 { "realms", realm, "kdc", NULL };
 
             const char * sec_admin[] =
@@ -718,7 +738,10 @@ k5_write_config_data(k5_config_data * d) {
                             pprofile_add_relation(profile, sec_admin,
                                                   host);
 
-                        d->realms[r].kdcs[k].flags &= ~K5_RKFLAG_NEW;
+                        d->realms[r].kdcs[k].flags &= ~ (K5_RKFLAG_NEW |
+                                                         K5_RKFLAG_MOD_MASTER |
+                                                         K5_RKFLAG_MOD_ADMIN);
+                        applied = TRUE;
                     }
                 }
 
@@ -737,6 +760,7 @@ k5_write_config_data(k5_config_data * d) {
                            is deleted and not in the profile file
                            anymore. */
                         d->realms[r].domain_maps[m].flags |= K5_DMFLAG_NEW;
+                        applied = TRUE;
                     } else if (!(d->realms[r].domain_maps[m].flags &
                                K5_DMFLAG_DELETED) &&
                              (d->realms[r].domain_maps[m].flags &
@@ -745,10 +769,12 @@ k5_write_config_data(k5_config_data * d) {
                                               realm);
 
                         d->realms[r].domain_maps[m].flags &= ~K5_DMFLAG_NEW;
+                        applied = TRUE;
                     }
                 }
 
-                d->realms[r].flags &= ~K5_RDFLAG_NEW;
+                d->realms[r].flags &= ~(K5_RDFLAG_NEW |
+                                        K5_RDFLAG_MODIFED);
 
             } else if ((d->realms[r].flags & K5_RDFLAG_DELETED) &&
                        !(d->realms[r].flags & K5_RDFLAG_NEW)) {
@@ -766,7 +792,10 @@ k5_write_config_data(k5_config_data * d) {
                         sec_all[2] = values[v];
                         pprofile_clear_relation(profile, sec_all);
                     }
+                    sec_all[2] = NULL;
+                    pprofile_rename_section(profile, sec_all, NULL);
                     pprofile_free_list(values);
+                    applied = TRUE;
                 }
 
                 rv = pprofile_get_relation_names(profile, sec_domain_realm,
@@ -782,10 +811,11 @@ k5_write_config_data(k5_config_data * d) {
 
                         if (!rv) {
                             if (!strcmp(maprealm, realm)) {
-                                StringCbCopyA(host, sizeof(host), 
+                                StringCbCopyA(host, sizeof(host),
                                               values[v]);
-                                pprofile_clear_relation(profile, 
+                                pprofile_clear_relation(profile,
                                                         sec_domain_map);
+                                applied = TRUE;
                             }
                             pprofile_release_string(maprealm);
                         }
@@ -798,7 +828,8 @@ k5_write_config_data(k5_config_data * d) {
                    deleted and is not in the profile file. */
                 d->realms[r].flags |= K5_RDFLAG_NEW;
 
-            } else if (!(d->realms[r].flags & K5_RDFLAG_DELETED)) {
+            } else if (!(d->realms[r].flags & K5_RDFLAG_DELETED) &&
+                       (d->realms[r].flags & K5_RDFLAG_MODIFED)) {
                 khm_size k;
                 khm_size m;
 
@@ -822,6 +853,8 @@ k5_write_config_data(k5_config_data * d) {
                         pprofile_update_relation(profile, sec_master,
                                                  host, NULL);
 
+                        applied = TRUE;
+
                         /* as above, setting 'new' flag to indicate
                            that the item does not exist in the profile
                            file. */
@@ -841,7 +874,10 @@ k5_write_config_data(k5_config_data * d) {
                             pprofile_add_relation(profile, sec_admin,
                                                   host);
 
-                        d->realms[r].kdcs[k].flags &= ~K5_RKFLAG_NEW;
+                        d->realms[r].kdcs[k].flags &= ~(K5_RKFLAG_NEW |
+                                                        K5_RKFLAG_MOD_ADMIN |
+                                                        K5_RKFLAG_MOD_MASTER);
+                        applied = TRUE;
                         continue;
                     }
 
@@ -856,6 +892,7 @@ k5_write_config_data(k5_config_data * d) {
                                                   host);
                         }
 
+                        applied = TRUE;
                         d->realms[r].kdcs[k].flags &= ~K5_RKFLAG_MOD_MASTER;
                     }
 
@@ -870,6 +907,7 @@ k5_write_config_data(k5_config_data * d) {
                                                      host, NULL);
                         }
 
+                        applied = TRUE;
                         d->realms[r].kdcs[k].flags &= ~K5_RKFLAG_MOD_ADMIN;
                     }
                 }
@@ -887,6 +925,7 @@ k5_write_config_data(k5_config_data * d) {
 
                         pprofile_clear_relation(profile, sec_domain_map);
                         d->realms[r].domain_maps[m].flags |= K5_DMFLAG_NEW;
+                        applied = TRUE;
 
                     } else if (!(d->realms[r].domain_maps[m].flags &
                                K5_DMFLAG_DELETED) &&
@@ -897,6 +936,8 @@ k5_write_config_data(k5_config_data * d) {
                         pprofile_add_relation(profile, sec_domain_map,
                                               realm);
                         d->realms[r].domain_maps[m].flags &= ~K5_DMFLAG_NEW;
+                        applied = TRUE;
+
                     }
                 }
 
@@ -932,11 +973,13 @@ k5_write_config_data(k5_config_data * d) {
         khui_alert_set_title(alert, title);
         khui_alert_set_message(alert, msg);
         khui_alert_set_suggestion(alert, sugg);
-        
+
         khui_alert_show(alert);
     }
 
     d->flags = 0;
+
+    return applied;
 }
 
 /* actual dialog stuff */
@@ -949,7 +992,7 @@ k5_write_config_data(k5_config_data * d) {
 static k5_config_data k5_config_dlg_data;
 static khm_boolean    k5_dlg_data_valid = FALSE;
 
-INT_PTR CALLBACK 
+INT_PTR CALLBACK
 k5_config_dlgproc(HWND hwnd,
                   UINT uMsg,
                   WPARAM wParam,
@@ -993,10 +1036,15 @@ k5_config_dlgproc(HWND hwnd,
                             (LPARAM) d->realms[i].realm);
             }
 
-            SendMessage(hw, CB_SELECTSTRING, -1,
+            SendMessage(hw, CB_SELECTSTRING, (WPARAM) -1,
                         (LPARAM) d->def_realm);
+            SetDlgItemText(hwnd, IDC_CFG_DEFREALM, d->def_realm);
+            SendDlgItemMessage(hwnd, IDC_CFG_DEFREALM, CB_LIMITTEXT,
+                               ARRAYLENGTH(d->def_realm) - 1, 0);
 
             SetDlgItemText(hwnd, IDC_CFG_CFGFILE, d->config_file);
+            SendDlgItemMessage(hwnd, IDC_CFG_CFGFILE, EM_LIMITTEXT,
+                               ARRAYLENGTH(d->config_file) - 1, 0);
 
             /* hostname/domain */
             if (NetWkstaGetInfo(NULL, 100, (LPBYTE *) &winfo100) == NERR_Success) {
@@ -1025,7 +1073,7 @@ k5_config_dlgproc(HWND hwnd,
 #endif
             SendMessage(hw, CB_RESETCONTENT, 0, 0);
 
-            for (t=importopts; 
+            for (t=importopts;
                  t && *t && *t != L' ' &&
                      t < importopts + ARRAYLENGTH(importopts);
                  t = multi_string_next(t)) {
@@ -1036,7 +1084,7 @@ k5_config_dlgproc(HWND hwnd,
             SendMessage(hw, CB_SETCURSEL, 0, d->lsa_import);
             t = importopts;
             SendMessage(hw, CB_GETLBTEXT, d->lsa_import,(LPARAM) t);
-            SendMessage(hw, CB_SELECTSTRING, -1, (LPARAM) t);
+            SendMessage(hw, CB_SELECTSTRING, (WPARAM) -1, (LPARAM) t);
         }
         break;
 
@@ -1046,6 +1094,9 @@ k5_config_dlgproc(HWND hwnd,
 
             d = &k5_config_dlg_data;
 
+            if (d == NULL)
+                return FALSE;
+
             if (wParam == MAKEWPARAM(IDC_CFG_IMPORT, CBN_SELCHANGE)) {
                 int idx;
                 int modified = FALSE;
@@ -1078,6 +1129,100 @@ k5_config_dlgproc(HWND hwnd,
                                    KHUI_CNFLAG_MODIFIED);
                 return TRUE;
             }
+
+            if (wParam == MAKEWPARAM(IDC_CFG_DEFREALM, CBN_EDITCHANGE)) {
+                wchar_t defrealm[K5_MAXCCH_REALM];
+                int t;
+
+                t = GetDlgItemText(hwnd, IDC_CFG_DEFREALM,
+                                   defrealm, ARRAYLENGTH(defrealm));
+                if (t == 0) {
+                    /* we failed to get the default realm from the
+                       control for some reason. */
+                    SetDlgItemText(hwnd, IDC_CFG_DEFREALM, L"");
+                    StringCbCopy(d->def_realm, sizeof(d->def_realm),
+                                 L"");
+                } else {
+                    StringCbCopy(d->def_realm, sizeof(d->def_realm),
+                                 defrealm);
+                }
+
+                d->flags |= K5_CDFLAG_MOD_DEF_REALM;
+
+                khui_cfg_set_flags(d->node_main,
+                                   KHUI_CNFLAG_MODIFIED,
+                                   KHUI_CNFLAG_MODIFIED);
+                return TRUE;
+            }
+
+            if (wParam == MAKEWPARAM(IDC_CFG_DEFREALM, CBN_SELCHANGE)) {
+                wchar_t defrealm[K5_MAXCCH_REALM];
+                LRESULT cursel, lr;
+
+                cursel = SendDlgItemMessage(hwnd, IDC_CFG_DEFREALM, CB_GETCURSEL,
+                                            0, 0);
+                if (cursel == CB_ERR)
+                    return TRUE;
+
+                lr = SendDlgItemMessage(hwnd, IDC_CFG_DEFREALM, CB_GETLBTEXTLEN,
+                                        cursel, 0);
+#ifdef DEBUG
+                assert(lr < ARRAYLENGTH(defrealm));
+#endif
+                if (lr >= ARRAYLENGTH(defrealm)) {
+                    /* we really shouldn't have any string here that
+                       exceeds that many characters.  But if we do, we
+                       ignore that since we don't consider it
+                       valid. */
+                    return TRUE;
+                }
+
+                lr = SendDlgItemMessage(hwnd, IDC_CFG_DEFREALM, CB_GETLBTEXT,
+                                        cursel, (LPARAM) defrealm);
+                if (lr == CB_ERR) {
+                    /* somehow we failed to copy the value anyway
+                       after all those checks.  */
+#ifdef DEBUG
+                    assert(FALSE);
+#endif
+                    return TRUE;
+                }
+
+                StringCbCopy(d->def_realm, sizeof(d->def_realm),
+                             defrealm);
+
+                d->flags |= K5_CDFLAG_MOD_DEF_REALM;
+
+                khui_cfg_set_flags(d->node_main,
+                                   KHUI_CNFLAG_MODIFIED,
+                                   KHUI_CNFLAG_MODIFIED);
+
+                return TRUE;
+            }
+
+#ifdef ALLOW_CHANGING_KRB5_CONFIG_FILE
+            if (wParam == MAKEWPARAM(IDC_CFG_CFGFILE, EN_CHANGE)) {
+                wchar_t cfgfile[MAX_PATH];
+                int t;
+
+                t = GetDlgItemText(hwnd, IDC_CFG_CFGFILE,
+                                   cfgfile, ARRAYLENGTH(cfgfile));
+
+                if (t == 0) {
+                    StringCbCopy(d->config_file, sizeof(d->config_file),
+                                 L"");
+                } else {
+                    StringCbCopy(d->config_file, sizeof(d->config_file),
+                                 cfgfile);
+                }
+
+                d->flags |= K5_CDFLAG_MOD_CONF_FILE;
+
+                khui_cfg_set_flags(d->node_main,
+                                   KHUI_CNFLAG_MODIFIED,
+                                   KHUI_CNFLAG_MODIFIED);
+            }
+#endif
         }
         break;
 
@@ -1089,13 +1234,14 @@ k5_config_dlgproc(HWND hwnd,
 
             if (HIWORD(wParam) == WMCFG_APPLY) {
                 khm_int32 oflags;
+                int applied;
 
                 oflags = d->flags;
-                k5_write_config_data(d);
+                applied = k5_write_config_data(d);
 
                 if (d->flags != oflags) {
                     khui_cfg_set_flags(d->node_main,
-                                       KHUI_CNFLAG_APPLIED,
+                                       (applied ? KHUI_CNFLAG_APPLIED : 0),
                                        KHUI_CNFLAG_APPLIED |
                                        KHUI_CNFLAG_MODIFIED);
                 }
@@ -1461,7 +1607,7 @@ k5_delete_realms(HWND hwnd, k5_config_data * d) {
         lvi.iItem = idx;
         lvi.iSubItem = 0;
         lvi.mask = LVIF_PARAM;
-        
+
         ListView_GetItem(hw_rlm, &lvi);
 
         if (lvi.lParam != -1 &&
@@ -1473,7 +1619,7 @@ k5_delete_realms(HWND hwnd, k5_config_data * d) {
 
     if (modified) {
         d->flags |= K5_CDFLAG_MOD_REALMS;
-        
+
         k5_purge_config_data(d, TRUE, TRUE, TRUE);
         k5_update_realms_display(hw_rlm, d);
         k5_update_dmap_display(GetDlgItem(hwnd, IDC_CFG_DMAP), NULL, 0);
@@ -1492,7 +1638,7 @@ k5_delete_servers(HWND hwnd, k5_config_data * d) {
 
     hw_kdc = GetDlgItem(hwnd, IDC_CFG_KDC);
     r = d->c_realm;
-    
+
     idx = -1;
     while((idx = ListView_GetNextItem(hw_kdc, idx,
                                       LVNI_SELECTED))
@@ -1514,7 +1660,7 @@ k5_delete_servers(HWND hwnd, k5_config_data * d) {
     if (modified) {
         d->flags |= K5_CDFLAG_MOD_REALMS;
         d->realms[r].flags |= K5_RDFLAG_MODIFED;
-        
+
         k5_purge_config_data(d, TRUE, TRUE, TRUE);
         k5_update_realms_display(GetDlgItem(hwnd, IDC_CFG_REALMS), d);
         k5_update_kdcs_display(hw_kdc, d, r);
@@ -1541,7 +1687,7 @@ k5_delete_dmap(HWND hwnd, k5_config_data * d) {
         lvi.iItem = idx;
         lvi.iSubItem = 0;
         lvi.mask = LVIF_PARAM;
-        
+
         ListView_GetItem(hw_dmp, &lvi);
 
         if (lvi.lParam != -1 &&
@@ -1554,7 +1700,7 @@ k5_delete_dmap(HWND hwnd, k5_config_data * d) {
     if (modified) {
         d->flags |= K5_CDFLAG_MOD_REALMS;
         k5_purge_config_data(d, FALSE, FALSE, TRUE);
-        
+
         if (!(d->realms[r].flags & K5_RDFLAG_MODIFED)) {
             d->realms[r].flags |= K5_RDFLAG_MODIFED;
 
@@ -1565,7 +1711,7 @@ k5_delete_dmap(HWND hwnd, k5_config_data * d) {
     }
 }
 
-INT_PTR CALLBACK 
+INT_PTR CALLBACK
 k5_realms_dlgproc(HWND hwnd,
                   UINT uMsg,
                   WPARAM wParam,
@@ -1700,7 +1846,7 @@ k5_realms_dlgproc(HWND hwnd,
 
                     SetMenuItemInfo(hm, CMD_DEL_REALM, FALSE, &mii);
                 }
-                
+
             } else if (id == IDC_CFG_KDC) {
                 HWND hw_kdc;
                 int n;
@@ -1737,7 +1883,7 @@ k5_realms_dlgproc(HWND hwnd,
                     mii.fState = MFS_ENABLED;
 
                     SetMenuItemInfo(hm, CMD_DEL_SERVER, FALSE, &mii);
-                    
+
                     mii.fState = MFS_DISABLED;
 
                     SetMenuItemInfo(hm, CMD_MAKE_ADMIN, FALSE, &mii);
@@ -2416,7 +2562,7 @@ k5_realms_dlgproc(HWND hwnd,
 
                 if (ListView_GetSelectedCount(hw_kdc) != 1)
                     return TRUE;
-                
+
                 idx = -1;
                 while ((idx = ListView_GetNextItem(hw_kdc, idx,
                                                    LVNI_SELECTED)) != -1) {
@@ -2467,7 +2613,7 @@ k5_realms_dlgproc(HWND hwnd,
 
                 if (ListView_GetSelectedCount(hw_kdc) != 1)
                     return TRUE;
-                
+
                 idx = -1;
                 while ((idx = ListView_GetNextItem(hw_kdc, idx,
                                                    LVNI_SELECTED)) != -1) {
@@ -2554,12 +2700,11 @@ k5_realms_dlgproc(HWND hwnd,
 
     case KHUI_WM_CFG_NOTIFY:
         /* the realms dialog receives this notification after the top
-           level krb5 configuration panel has received it.  Therefore,
-           we assume that any changes have already been applied.  When
-           applying changes, we switch the mod bits off to indicate
-           that the changes have been written.  We just have to
-           repaint the screen at this point. */
+           level krb5 configuration panel has received it. */
         if (HIWORD(wParam) == WMCFG_APPLY) {
+            int applied;
+
+            applied = k5_write_config_data(d);
             k5_purge_config_data(d, TRUE, TRUE, TRUE);
             k5_update_realms_display(GetDlgItem(hwnd, IDC_CFG_REALMS), d);
             if (d->c_realm != -1) {
@@ -2569,6 +2714,10 @@ k5_realms_dlgproc(HWND hwnd,
                 k5_update_kdcs_display(GetDlgItem(hwnd, IDC_CFG_KDC), NULL, 0);
                 k5_update_dmap_display(GetDlgItem(hwnd, IDC_CFG_DMAP), NULL, 0);
             }
+            khui_cfg_set_flags(d->node_realm,
+                               (applied ? KHUI_CNFLAG_APPLIED : 0),
+                               KHUI_CNFLAG_APPLIED);
+
         }
         break;
     }
@@ -2701,7 +2850,7 @@ k5_unregister_config_panels(void) {
 #endif
     }
 
-    if (KHM_SUCCEEDED(khui_cfg_open(node_main, L"KerberosRealms", 
+    if (KHM_SUCCEEDED(khui_cfg_open(node_main, L"KerberosRealms",
                                     &node_realms))) {
         khui_cfg_remove(node_realms);
         khui_cfg_release(node_realms);
