@@ -49,15 +49,52 @@ typedef struct tag_k5_new_cred_data {
     HWND hw_realm;
 } k5_new_cred_data;
 
+static
+void
+trim_str(wchar_t * s, khm_size cch) {
+    wchar_t * c, * last_ws;
+
+    for (c = s; *c && iswspace(*c) && ((khm_size)(c - s)) < cch; c++);
+
+    if (((khm_size)(c - s)) >= cch)
+        return;
+
+    if (c != s && ((khm_size)(c - s)) < cch) {
+#if _MSC_VER >= 1400 && __STDC_WANT_SECURE_LIB__
+        wmemmove_s(s, cch, c, cch - ((khm_size)(c - s)));
+#else
+        memmove(s, c, (cch - ((khm_size)(c - s))) * sizeof(wchar_t));
+#endif
+    }
+
+    last_ws = NULL;
+    for (c = s; *c && ((khm_size)(c - s)) < cch; c++) {
+        if (!iswspace(*c))
+            last_ws = NULL;
+        else if (last_ws == NULL)
+            last_ws = c;
+    }
+
+    if (last_ws)
+        *last_ws = L'\0';
+}
+
 /* Runs in the UI thread */
-int 
-k5_get_realm_from_nc(khui_new_creds * nc, 
-                     wchar_t * buf, 
+int
+k5_get_realm_from_nc(khui_new_creds * nc,
+                     wchar_t * buf,
                      khm_size cch_buf) {
     k5_new_cred_data * d;
+    khm_size s;
 
     d = (k5_new_cred_data *) nc->ident_aux;
-    return GetWindowText(d->hw_realm, buf, (int) cch_buf);
+    buf[0] = L'\0';
+    GetWindowText(d->hw_realm, buf, (int) cch_buf);
+    trim_str(buf, cch_buf);
+
+    StringCchLength(buf, cch_buf, &s);
+
+    return (int) s;
 }
 
 /* set the primary identity of a new credentials dialog depending on
@@ -65,7 +102,7 @@ k5_get_realm_from_nc(khui_new_creds * nc,
 
    Runs in the UI thread
 */
-static void 
+static void
 set_identity_from_ui(khui_new_creds * nc,
                      k5_new_cred_data * d) {
     wchar_t un[KCDB_IDENT_MAXCCH_NAME];
@@ -83,6 +120,7 @@ set_identity_from_ui(khui_new_creds * nc,
     assert(cch < KCDB_IDENT_MAXCCH_NAME - 1);
 
     GetWindowText(d->hw_username, un, ARRAYLENGTH(un));
+    trim_str(un, ARRAYLENGTH(un));
 
     realm = khm_get_realm_from_princ(un);
     if (realm)          /* realm was specified */
@@ -113,6 +151,7 @@ set_identity_from_ui(khui_new_creds * nc,
     }
 
     GetWindowText(d->hw_realm, realm, (int) cch_left);
+    trim_str(realm, cch_left);
 
  _set_ident:
     if (KHM_FAILED(rv = kcdb_identity_create(un,
@@ -183,11 +222,14 @@ update_crossfeed(khui_new_creds * nc,
     GetWindowText(d->hw_username,
                   un,
                   ARRAYLENGTH(un));
+    trim_str(un, ARRAYLENGTH(un));
 
     un_realm = khm_get_realm_from_princ(un);
 
-    if (un_realm == NULL)
+    if (un_realm == NULL) {
+        EnableWindow(d->hw_realm, TRUE);
         return FALSE;
+    }
 
     if (ctrl_id_src == K5_NCID_UN) {
 
@@ -230,6 +272,15 @@ update_crossfeed(khui_new_creds * nc,
         SetWindowText(d->hw_realm,
                       un_realm);
 
+        if (GetFocus() == d->hw_realm) {
+            HWND hw_next = GetNextDlgTabItem(nc->hwnd, d->hw_realm,
+                                             FALSE);
+            if (hw_next)
+                SetFocus(hw_next);
+        }
+
+        EnableWindow(d->hw_realm, FALSE);
+
         return TRUE;
     }
     /* else... */
@@ -246,6 +297,7 @@ update_crossfeed(khui_new_creds * nc,
 
     GetWindowText(d->hw_realm, realm,
                   ARRAYLENGTH(realm));
+    trim_str(realm, ARRAYLENGTH(realm));
 
     idx = (int)SendMessage(d->hw_realm,
                            CB_FINDSTRINGEXACT,
@@ -276,13 +328,13 @@ update_crossfeed(khui_new_creds * nc,
 
     SetWindowText(d->hw_username, un);
 
-    return TRUE;    
+    return TRUE;
 }
 
 /* Handle window messages for the identity specifiers
 
    runs in UI thread */
-static LRESULT 
+static LRESULT
 handle_wnd_msg(khui_new_creds * nc,
                HWND hwnd,
                UINT uMsg,
@@ -299,7 +351,7 @@ handle_wnd_msg(khui_new_creds * nc,
             /* the username has changed.  Instead of handling this
                for every keystroke, set a timer that elapses some
                time afterwards and then handle the event. */
-            SetTimer(hwnd, NC_UNCHANGE_TIMER, 
+            SetTimer(hwnd, NC_UNCHANGE_TIMER,
                      NC_UNCHANGE_TIMEOUT, NULL);
             return TRUE;
 
@@ -348,7 +400,7 @@ handle_wnd_msg(khui_new_creds * nc,
 /* UI Callback
 
    runs in UI thread */
-static LRESULT KHMAPI 
+static LRESULT KHMAPI
 ui_cb(khui_new_creds * nc,
       UINT cmd,
       HWND hwnd,
@@ -391,7 +443,7 @@ ui_cb(khui_new_creds * nc,
             nc->ident_aux = (LPARAM) d;
             khui_cw_unlock_nc(nc);
 
-            LoadString(hResModule, IDS_NC_USERNAME, 
+            LoadString(hResModule, IDS_NC_USERNAME,
                        wbuf, ARRAYLENGTH(wbuf));
 
             d->hw_username_label = CreateWindow
@@ -408,8 +460,8 @@ ui_cb(khui_new_creds * nc,
             d->hw_username = CreateWindow
                 (L"COMBOBOX",
                  L"",
-                 CBS_DROPDOWN | CBS_AUTOHSCROLL | CBS_SORT | 
-                 WS_CHILD | WS_VISIBLE | WS_TABSTOP,
+                 CBS_DROPDOWN | CBS_AUTOHSCROLL | CBS_SORT |
+                 WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_VSCROLL,
                  0, 0, 100, 100, /* bogus values */
                  hw_parent,
                  (HMENU) K5_NCID_UN,
@@ -449,8 +501,8 @@ ui_cb(khui_new_creds * nc,
             d->hw_realm = CreateWindow
                 (L"COMBOBOX",
                  L"",
-                 CBS_DROPDOWN | CBS_AUTOHSCROLL | CBS_SORT | 
-                 WS_CHILD | WS_VISIBLE | WS_TABSTOP,
+                 CBS_DROPDOWN | CBS_AUTOHSCROLL | CBS_SORT |
+                 WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_VSCROLL,
                  0, 0, 100, 100, /* bogus */
                  hw_parent,
                  (HMENU) K5_NCID_REALM,
@@ -670,6 +722,17 @@ ui_cb(khui_new_creds * nc,
     case WMNC_IDENT_WMSG:
         return handle_wnd_msg(nc, hwnd, uMsg, wParam, lParam);
 
+    case WMNC_IDENT_PREPROCESS:
+        {
+#ifdef DEBUG
+            assert(d != NULL);
+#endif
+            if (d) {
+                set_identity_from_ui(nc, d);
+            }
+        }
+        return TRUE;
+
     case WMNC_IDENT_EXIT:
         {
 #ifdef DEBUG
@@ -678,7 +741,7 @@ ui_cb(khui_new_creds * nc,
             khui_cw_lock_nc(nc);
             nc->ident_aux = 0;
             khui_cw_unlock_nc(nc);
-            
+
             /* since we created all the windows as child windows of
                the new creds window, they will be destroyed when that
                window is destroyed. */
@@ -690,7 +753,7 @@ ui_cb(khui_new_creds * nc,
 }
 
 static khm_int32
-k5_ident_valiate_name(khm_int32 msg_type,
+k5_ident_validate_name(khm_int32 msg_type,
                       khm_int32 msg_subtype,
                       khm_ui_4 uparam,
                       void * vparam) {
@@ -698,6 +761,7 @@ k5_ident_valiate_name(khm_int32 msg_type,
     char princ_name[KCDB_IDENT_MAXCCH_NAME];
     kcdb_ident_name_xfer * nx;
     krb5_error_code code;
+    wchar_t * atsign;
 
     nx = (kcdb_ident_name_xfer *) vparam;
 
@@ -718,11 +782,18 @@ k5_ident_valiate_name(khm_int32 msg_type,
         return KHM_ERROR_SUCCESS;
     }
 
-    if (princ != NULL) 
+    if (princ != NULL)
         pkrb5_free_principal(k5_identpro_ctx,
                              princ);
 
-    nx->result = KHM_ERROR_SUCCESS;
+    /* krb5_parse_name() accepts principal names with no realm or an
+       empty realm.  We don't. */
+    atsign = wcschr(nx->name_src, L'@');
+    if (atsign == NULL || atsign[1] == L'\0') {
+        nx->result = KHM_ERROR_INVALID_NAME;
+    } else {
+        nx->result = KHM_ERROR_SUCCESS;
+    }
 
     return KHM_ERROR_SUCCESS;
 }
@@ -741,155 +812,118 @@ k5_update_last_default_identity(khm_handle ident) {
     khc_write_string(csp_params, L"LastDefaultIdent", idname);
 }
 
+static khm_int32
+k5_ident_set_default_int(khm_handle def_ident) {
+    wchar_t id_ccname[KRB5_MAXCCH_CCNAME];
+    khm_size cb;
+    DWORD dw;
+    LONG l;
+    HKEY hk_ccname;
+    DWORD dwType;
+    DWORD dwSize;
+    wchar_t reg_ccname[KRB5_MAXCCH_CCNAME];
+
+#ifdef DEBUG
+    assert(def_ident != NULL);
+#endif
+
+    cb = sizeof(id_ccname);
+    if (KHM_FAILED(kcdb_identity_get_attr(def_ident, attr_id_krb5_ccname, NULL,
+                                          id_ccname, &cb))) {
+        _reportf(L"The specified identity does not have the Krb5CCName property");
+
+        cb = sizeof(id_ccname);
+        if (KHM_FAILED(khm_krb5_get_identity_default_ccache(def_ident, id_ccname, &cb))) {
+            return KHM_ERROR_INVALID_PARAM;
+        }
+    }
+
+    _reportf(L"Found Krb5CCName property : %s", id_ccname);
+
+    StringCbLength(id_ccname, sizeof(id_ccname), &cb);
+    cb += sizeof(wchar_t);
+
+    _reportf(L"Setting default CC name in the registry");
+
+    l = RegOpenKeyEx(HKEY_CURRENT_USER, L"Software\\MIT\\kerberos5", 0,
+                     KEY_READ | KEY_WRITE, &hk_ccname);
+
+    if (l != ERROR_SUCCESS)
+        l = RegCreateKeyEx(HKEY_CURRENT_USER, L"Software\\MIT\\kerberos5", 0,
+                           NULL, REG_OPTION_NON_VOLATILE, KEY_READ | KEY_WRITE,
+                           NULL, &hk_ccname, &dw);
+
+    if (l != ERROR_SUCCESS) {
+        _reportf(L"Can't create registry key : %d", l);
+        _end_task();
+        return KHM_ERROR_UNKNOWN;
+    }
+
+    dwSize = sizeof(reg_ccname);
+
+    l = RegQueryValueEx(hk_ccname, L"ccname", NULL, &dwType, (LPBYTE) reg_ccname,
+                        &dwSize);
+
+    if (l != ERROR_SUCCESS ||
+        dwType != REG_SZ ||
+        khm_krb5_cc_name_cmp(reg_ccname, id_ccname)) {
+
+        /* we have to write the new value in */
+
+        l = RegSetValueEx(hk_ccname, L"ccname", 0, REG_SZ, (BYTE *) id_ccname,
+                          (DWORD) cb);
+    }
+
+    RegCloseKey(hk_ccname);
+
+    if (l == ERROR_SUCCESS) {
+        _reportf(L"Successfully set the default ccache");
+        k5_update_last_default_identity(def_ident);
+        return KHM_ERROR_SUCCESS;
+    } else {
+        _reportf(L"Can't set the registry value : %d", l);
+        return KHM_ERROR_UNKNOWN;
+    }
+}
+
 static khm_int32
 k5_ident_set_default(khm_int32 msg_type,
                      khm_int32 msg_subtype,
                      khm_ui_4 uparam,
                      void * vparam) {
 
-    /* Logic for setting the default identity:
-
-    When setting identity I as the default;
-
-    - If KRB5CCNAME is set
-    - If I["Krb5CCName"] == %KRB5CCNAME%
-    - do nothing
-    - Else
-    - Copy the contents of I["Krb5CCName"] to %KRB5CCNAME
-    - Set I["Krb5CCName"] to %KRB5CCNAME
-    - Else
-    - Set HKCU\Software\MIT\kerberos5,ccname to 
-    "API:".I["Krb5CCName"]
+    /*
+       Currently, setting the default identity simply sets the
+       "ccname" registry value at "Software\MIT\kerberos5".
     */
 
     if (uparam) {
         /* an identity is being made default */
         khm_handle def_ident = (khm_handle) vparam;
-        wchar_t env_ccname[KRB5_MAXCCH_CCNAME];
-        wchar_t id_ccname[KRB5_MAXCCH_CCNAME];
-        khm_size cb;
-        DWORD dw;
-        LONG l;
+        khm_int32 rv;
 
 #ifdef DEBUG
         assert(def_ident != NULL);
 #endif
 
-        cb = sizeof(id_ccname);
-        if (KHM_FAILED(kcdb_identity_get_attr(def_ident,
-                                              attr_id_krb5_ccname,
-                                              NULL,
-                                              id_ccname,
-                                              &cb)))
-            return KHM_ERROR_UNKNOWN;
+        {
+            wchar_t idname[KCDB_IDENT_MAXCCH_NAME];
+            khm_size cb;
 
-        khm_krb5_canon_cc_name(id_ccname, sizeof(id_ccname));
+            cb = sizeof(idname);
+            kcdb_identity_get_name(def_ident, idname, &cb);
 
-        StringCbLength(id_ccname, sizeof(id_ccname), &cb);
-        cb += sizeof(wchar_t);
-
-        dw = GetEnvironmentVariable(L"KRB5CCNAME",
-                                    env_ccname,
-                                    ARRAYLENGTH(env_ccname));
-
-        if (dw == 0 &&
-            GetLastError() == ERROR_ENVVAR_NOT_FOUND) {
-            /* KRB5CCNAME not set */
-            HKEY hk_ccname;
-            DWORD dwType;
-            DWORD dwSize;
-            wchar_t reg_ccname[KRB5_MAXCCH_CCNAME];
-
-            l = RegOpenKeyEx(HKEY_CURRENT_USER,
-                             L"Software\\MIT\\kerberos5",
-                             0,
-                             KEY_READ | KEY_WRITE,
-                             &hk_ccname);
-
-            if (l != ERROR_SUCCESS)
-                l = RegCreateKeyEx(HKEY_CURRENT_USER,
-                                   L"Software\\MIT\\kerberos5",
-                                   0,
-                                   NULL,
-                                   REG_OPTION_NON_VOLATILE,
-                                   KEY_READ | KEY_WRITE,
-                                   NULL,
-                                   &hk_ccname,
-                                   &dw);
-
-            if (l != ERROR_SUCCESS)
-                return KHM_ERROR_UNKNOWN;
-
-            dwSize = sizeof(reg_ccname);
-
-            l = RegQueryValueEx(hk_ccname,
-                                L"ccname",
-                                NULL,
-                                &dwType,
-                                (LPBYTE) reg_ccname,
-                                &dwSize);
-
-            if (l != ERROR_SUCCESS ||
-                dwType != REG_SZ ||
-                khm_krb5_cc_name_cmp(reg_ccname, id_ccname)) {
-
-                /* we have to write the new value in */
-
-                l = RegSetValueEx(hk_ccname,
-                                  L"ccname",
-                                  0,
-                                  REG_SZ,
-                                  (BYTE *) id_ccname,
-                                  (DWORD) cb);
-            }
-
-            RegCloseKey(hk_ccname);
-
-            if (l == ERROR_SUCCESS) {
-                k5_update_last_default_identity(def_ident);
-                return KHM_ERROR_SUCCESS;
-            } else
-                return KHM_ERROR_UNKNOWN;
-
-        } else if (dw > ARRAYLENGTH(env_ccname)) {
-            /* buffer was not enough */
-#ifdef DEBUG
-            assert(FALSE);
-#else
-            return KHM_ERROR_UNKNOWN;
-#endif
-        } else {
-            /* KRB5CCNAME is set */
-            long code;
-            krb5_context ctx;
-
-            /* if the %KRB5CCNAME is the same as the identity
-               ccache, then it is already the default. */
-            if (!khm_krb5_cc_name_cmp(id_ccname, env_ccname)) {
-                k5_update_last_default_identity(def_ident);
-                return KHM_ERROR_SUCCESS;
-            }
-
-            /* if not, we have to copy the contents of id_ccname
-               to env_ccname */
-            code = pkrb5_init_context(&ctx);
-            if (code)
-                return KHM_ERROR_UNKNOWN;
-
-            code = khm_krb5_copy_ccache_by_name(ctx, 
-                                                env_ccname, 
-                                                id_ccname);
-
-            if (code == 0) {
-                k5_update_last_default_identity(def_ident);
-                khm_krb5_list_tickets(&ctx);
-            }
-
-            if (ctx)
-                pkrb5_free_context(ctx);
-
-            return (code == 0)?KHM_ERROR_SUCCESS:KHM_ERROR_UNKNOWN;
+            _begin_task(0);
+            _report_cs1(KHERR_DEBUG_1, L"Setting default identity [%1!s!]", _cstr(idname));
+            _describe();
         }
+
+        rv = k5_ident_set_default_int(def_ident);
+
+        _end_task();
+
+        return rv;
+
     } else {
         /* the default identity is being forgotten */
 
@@ -991,15 +1025,28 @@ k5_ident_notify_create(khm_int32 msg_type,
     return KHM_ERROR_SUCCESS;
 }
 
+struct k5_ident_update_data {
+    khm_handle identity;
+
+    FILETIME   ft_expire;      /* expiration */
+    FILETIME   ft_issue;       /* issue */
+    FILETIME   ft_rexpire;     /* renew expiration */
+    wchar_t    ccname[KRB5_MAXCCH_CCNAME];
+    khm_int32  k5_flags;
+};
+
+/* The logic here has to reflect the logic in khm_krb5_list_tickets().
+   We use this to handle an identity update request because some other
+   plug-in or maybe NetIDMgr itself is about to do something
+   important(tm) with the identity and needs to make sure that the
+   properties of the identity are up-to-date. */
 static khm_int32 KHMAPI
 k5_ident_update_apply_proc(khm_handle cred,
                            void * rock) {
-    wchar_t ccname[KRB5_MAXCCH_CCNAME];
-    khm_handle tident = (khm_handle) rock;
+    struct k5_ident_update_data * d = (struct k5_ident_update_data *) rock;
     khm_handle ident = NULL;
     khm_int32 t;
     khm_int32 flags;
-    FILETIME t_expire;
     FILETIME t_cexpire;
     FILETIME t_rexpire;
     khm_size cb;
@@ -1008,12 +1055,15 @@ k5_ident_update_apply_proc(khm_handle cred,
     if (KHM_FAILED(kcdb_cred_get_type(cred, &t)) ||
         t != credtype_id_krb5 ||
         KHM_FAILED(kcdb_cred_get_identity(cred, &ident)))
+
         return KHM_ERROR_SUCCESS;
 
-    if (!kcdb_identity_is_equal(ident,tident))
+    if (!kcdb_identity_is_equal(ident,d->identity))
+
         goto _cleanup;
 
     if (KHM_FAILED(kcdb_cred_get_flags(cred, &flags)))
+
         flags = 0;
 
     if (flags & KCDB_CRED_FLAG_INITIAL) {
@@ -1023,13 +1073,9 @@ k5_ident_update_apply_proc(khm_handle cred,
                                              NULL,
                                              &t_cexpire,
                                              &cb))) {
-            cb = sizeof(t_expire);
-            if (KHM_FAILED(kcdb_identity_get_attr(tident,
-                                                  KCDB_ATTR_EXPIRE,
-                                                  NULL,
-                                                  &t_expire,
-                                                  &cb)) ||
-                CompareFileTime(&t_cexpire, &t_expire) > 0) {
+            if ((d->ft_expire.dwLowDateTime == 0 &&
+                 d->ft_expire.dwHighDateTime == 0) ||
+                CompareFileTime(&t_cexpire, &d->ft_expire) > 0) {
                 goto update_identity;
             }
         }
@@ -1039,52 +1085,35 @@ k5_ident_update_apply_proc(khm_handle cred,
 
  update_identity:
 
-    kcdb_identity_set_attr(tident, KCDB_ATTR_EXPIRE,
-                           &t_cexpire, sizeof(t_cexpire));
+    d->ft_expire = t_cexpire;
 
-    cb = sizeof(ccname);
-    if (KHM_SUCCEEDED(kcdb_cred_get_attr(cred, KCDB_ATTR_LOCATION,
-                                         NULL,
-                                         ccname,
-                                         &cb))) {
-        kcdb_identity_set_attr(tident, attr_id_krb5_ccname,
-                               ccname, cb);
-    } else {
-        kcdb_identity_set_attr(tident, attr_id_krb5_ccname,
-                               NULL, 0);
-    }
-                
-    cb = sizeof(t);
-    if (KHM_SUCCEEDED(kcdb_cred_get_attr(cred,
-                                         attr_id_krb5_flags,
-                                         NULL,
-                                         &t,
-                                         &cb))) {
-
-        kcdb_identity_set_attr(tident, attr_id_krb5_flags, 
-                               &t, sizeof(t));
-
-        cb = sizeof(t_rexpire);
-        if (!(t & TKT_FLG_RENEWABLE) ||
-            KHM_FAILED(kcdb_cred_get_attr(cred,
-                                          KCDB_ATTR_RENEW_EXPIRE,
-                                          NULL,
-                                          &t_rexpire,
-                                          &cb))) {
-            kcdb_identity_set_attr(tident, KCDB_ATTR_RENEW_EXPIRE,
-                                   NULL, 0);
-        } else {
-            kcdb_identity_set_attr(tident, KCDB_ATTR_RENEW_EXPIRE,
-                                   &t_rexpire, sizeof(t_rexpire));
-        }
-    } else {
-        kcdb_identity_set_attr(tident, attr_id_krb5_flags,
-                               NULL, 0);
-        kcdb_identity_set_attr(tident, KCDB_ATTR_RENEW_EXPIRE,
-                               NULL, 0);
+    cb = sizeof(d->ccname);
+    if (KHM_FAILED(kcdb_cred_get_attr(cred, KCDB_ATTR_LOCATION, NULL, d->ccname, &cb))) {
+        d->ccname[0] = L'\0';
     }
 
-    rv = KHM_ERROR_EXIT;
+    cb = sizeof(d->k5_flags);
+    if (KHM_FAILED(kcdb_cred_get_attr(cred, attr_id_krb5_flags, NULL,
+                                         &d->k5_flags, &cb))) {
+        d->k5_flags = 0;
+    }
+
+    cb = sizeof(d->ft_issue);
+    if (KHM_FAILED(kcdb_cred_get_attr(cred, KCDB_ATTR_ISSUE, NULL, &d->ft_issue, &cb))) {
+        ZeroMemory(&d->ft_issue, sizeof(d->ft_issue));
+    }
+
+    cb = sizeof(t_rexpire);
+    if ((d->k5_flags & TKT_FLG_RENEWABLE) &&
+        KHM_SUCCEEDED(kcdb_cred_get_attr(cred,
+                                         KCDB_ATTR_RENEW_EXPIRE,
+                                         NULL,
+                                         &t_rexpire,
+                                         &cb))) {
+        d->ft_rexpire = t_rexpire;
+    } else {
+        ZeroMemory(&d->ft_rexpire, sizeof(d->ft_rexpire));
+    }
 
  _cleanup:
     if (ident)
@@ -1099,6 +1128,9 @@ k5_ident_update(khm_int32 msg_type,
                 khm_ui_4 uparam,
                 void * vparam) {
 
+#if 0
+    struct k5_ident_update_data d;
+#endif
     khm_handle ident;
     khm_handle tident;
     krb5_ccache cc = NULL;
@@ -1112,9 +1144,60 @@ k5_ident_update(khm_int32 msg_type,
     if (ident == NULL)
         return KHM_ERROR_SUCCESS;
 
+#if 0
+    /* we are going to skip doing this here since
+       khm_krb5_list_tickets() performs this function for us each time
+       we enumerate tickets.  Since it also gets run each time our
+       list of tickets changes and since we are basing this operation
+       on existing tickets, we are unlikely to find anything new
+       here.  */
+    ZeroMemory(&d, sizeof(d));
+    d.identity = ident;
+
     kcdb_credset_apply(NULL,
                        k5_ident_update_apply_proc,
-                       (void *) ident);
+                       (void *) &d);
+
+    if (d.ft_expire.dwLowDateTime != 0 ||
+        d.ft_expire.dwHighDateTime != 0) {
+
+        /* we found a TGT */
+
+        kcdb_identity_set_attr(ident, KCDB_ATTR_EXPIRE,
+                               &d.ft_expire, sizeof(d.ft_expire));
+        if (d.ft_issue.dwLowDateTime != 0 ||
+            d.ft_issue.dwHighDateTime != 0)
+            kcdb_identity_set_attr(ident, KCDB_ATTR_ISSUE,
+                                   &d.ft_issue, sizeof(d.ft_issue));
+        else
+            kcdb_identity_set_attr(ident, KCDB_ATTR_ISSUE, NULL, 0);
+
+        if (d.ft_rexpire.dwLowDateTime != 0 ||
+            d.ft_rexpire.dwHighDateTime != 0)
+            kcdb_identity_set_attr(ident, KCDB_ATTR_RENEW_EXPIRE,
+                                   &d.ft_rexpire, sizeof(d.ft_rexpire));
+        else
+            kcdb_identity_set_attr(ident, KCDB_ATTR_RENEW_EXPIRE, NULL, 0);
+
+        kcdb_identity_set_attr(ident, attr_id_krb5_flags,
+                               &d.k5_flags, sizeof(d.k5_flags));
+
+        if (d.ccname[0])
+            kcdb_identity_set_attr(ident, attr_id_krb5_ccname,
+                                   d.ccname, KCDB_CBSIZE_AUTO);
+        else
+            kcdb_identity_set_attr(ident, attr_id_krb5_ccname, NULL, 0);
+
+    } else {
+        /* Clear out the attributes.  We don't have any information
+           about this identity */
+        kcdb_identity_set_attr(ident, KCDB_ATTR_EXPIRE, NULL, 0);
+        kcdb_identity_set_attr(ident, KCDB_ATTR_ISSUE, NULL, 0);
+        kcdb_identity_set_attr(ident, KCDB_ATTR_RENEW_EXPIRE, NULL, 0);
+        kcdb_identity_set_attr(ident, attr_id_krb5_flags, NULL, 0);
+        kcdb_identity_set_attr(ident, attr_id_krb5_ccname, NULL, 0);
+    }
+#endif
 
     if (KHM_SUCCEEDED(kcdb_identity_get_default(&tident))) {
         kcdb_identity_release(tident);
@@ -1164,18 +1247,54 @@ k5_refresh_default_identity(krb5_context ctx) {
     krb5_principal princ = NULL;
     char * princ_nameA = NULL;
     wchar_t princ_nameW[KCDB_IDENT_MAXCCH_NAME];
+    char * ccname = NULL;
     khm_handle ident = NULL;
     khm_boolean found_default = FALSE;
 
     assert(ctx != NULL);
 
+    _begin_task(0);
+    _report_cs0(KHERR_DEBUG_1, L"Refreshing default identity");
+    _describe();
+
     code = pkrb5_cc_default(ctx, &cc);
-    if (code)
+    if (code) {
+        _reportf(L"Can't open default ccache. code=%d", code);
         goto _nc_cleanup;
-    
+    }
+
     code = pkrb5_cc_get_principal(ctx, cc, &princ);
-    if (code)
+    if (code) {
+        /* try to determine the identity from the ccache name */
+        ccname = pkrb5_cc_get_name(ctx, cc);
+
+        if (ccname) {
+            char * namepart = strchr(ccname, ':');
+
+            _reportf(L"CC name is [%S]", ccname);
+
+            if (namepart == NULL)
+                namepart = ccname;
+            else
+                namepart++;
+
+            _reportf(L"Checking if [%S] is a valid identity name", namepart);
+
+            AnsiStrToUnicode(princ_nameW, sizeof(princ_nameW), namepart);
+            if (kcdb_identity_is_valid_name(princ_nameW)) {
+                kcdb_identity_create(princ_nameW, KCDB_IDENT_FLAG_CREATE, &ident);
+                if (ident) {
+                    _reportf(L"Setting [%S] as the default identity", namepart);
+                    kcdb_identity_set_default_int(ident);
+                    found_default = TRUE;
+                }
+            }
+        } else {
+            _reportf(L"Can't determine ccache name");
+        }
+
         goto _nc_cleanup;
+    }
 
     code = pkrb5_unparse_name(ctx, princ, &princ_nameA);
     if (code)
@@ -1183,14 +1302,22 @@ k5_refresh_default_identity(krb5_context ctx) {
 
     AnsiStrToUnicode(princ_nameW, sizeof(princ_nameW), princ_nameA);
 
-    if (KHM_FAILED(kcdb_identity_create(princ_nameW, 0, &ident)))
-        goto _nc_cleanup;
+    _reportf(L"Found principal [%s]", princ_nameW);
 
+    if (KHM_FAILED(kcdb_identity_create(princ_nameW, KCDB_IDENT_FLAG_CREATE, &ident))) {
+        _reportf(L"Failed to create identity");
+        goto _nc_cleanup;
+    }
+
+    _reportf(L"Setting default identity to [%s]", princ_nameW);
     kcdb_identity_set_default_int(ident);
 
     found_default = TRUE;
 
  _nc_cleanup:
+
+    _end_task();
+
     if (princ_nameA)
         pkrb5_free_unparsed_name(ctx, princ_nameA);
 
@@ -1232,10 +1359,102 @@ k5_ident_init(khm_int32 msg_type,
             if (ident) {
                 kcdb_identity_set_default_int(ident);
                 kcdb_identity_release(ident);
+
+                found_default = TRUE;
             }
         }
     }
 
+    if (!found_default) {
+
+        /* There was no default ccache and we don't have a
+           "LastDefaultIdent" value. Next we see if there are any
+           identities that have credentials which have a Krb5CCName
+           property (i.e. an identity that has a Kerberos 5 TGT), and
+           make it the default.
+
+           Note that since the Krb5Ident plug-in has a dependency on
+           Krb5Cred, by the time this code runs, we already have a
+           listing of Kerberos 5 tickets and identities. */
+
+        wchar_t * idlist = NULL;
+        wchar_t * thisid;
+        khm_size cb = 0;
+        khm_size n_idents = 0;
+        khm_int32 rv;
+        wchar_t ccname[KRB5_MAXCCH_CCNAME];
+        FILETIME ft_expire;
+        FILETIME ft_now;
+        FILETIME ft_threshold;
+        BOOL match_all = FALSE;
+
+        rv = kcdb_identity_enum(0, 0, NULL, &cb, &n_idents);
+
+        TimetToFileTimeInterval(5 * 60, &ft_threshold);
+        GetSystemTimeAsFileTime(&ft_now);
+        ft_now = FtAdd(&ft_now, &ft_threshold);
+
+        while (rv == KHM_ERROR_TOO_LONG && n_idents > 0) {
+            if (idlist) {
+                PFREE(idlist);
+                idlist = NULL;
+            }
+
+            idlist = PMALLOC(cb);
+
+            if (idlist == NULL)
+                break;
+
+            rv = kcdb_identity_enum(0, 0, idlist, &cb, &n_idents);
+        }
+
+        if (KHM_SUCCEEDED(rv)) {
+
+            /* first we try to find an identity that has a valid TGT.
+               If that fails, then we try to find an identity with
+               *any* TGT. */
+
+        try_again:
+
+            for (thisid = idlist;
+                 thisid && *thisid && !found_default;
+                 thisid = multi_string_next(thisid)) {
+
+                if (KHM_SUCCEEDED(kcdb_identity_create(thisid, 0, &ident))) {
+                    khm_size cb_ft = sizeof(FILETIME);
+                    cb = sizeof(ccname);
+
+                    if (KHM_SUCCEEDED(kcdb_identity_get_attr(ident, attr_id_krb5_ccname,
+                                                             NULL, ccname, &cb)) &&
+                        (match_all ||
+                         (KHM_SUCCEEDED(kcdb_identity_get_attr(ident, KCDB_ATTR_EXPIRE,
+                                                               NULL, &ft_expire, &cb_ft)) &&
+                          CompareFileTime(&ft_expire, &ft_now) > 0))) {
+
+                        /* found one */
+                        k5_ident_set_default_int(ident);
+                        kcdb_identity_set_default_int(ident);
+                        found_default = TRUE;
+
+                    }
+
+                    kcdb_identity_release(ident);
+                    ident = NULL;
+                }
+            }
+
+            if (!found_default && !match_all) {
+                match_all = TRUE;
+                goto try_again;
+            }
+        }
+
+        if (idlist) {
+            PFREE(idlist);
+            idlist = NULL;
+        }
+    }
+
     return KHM_ERROR_SUCCESS;
 }
 
@@ -1280,10 +1499,10 @@ k5_ident_(khm_int32 msg_type,
 }
 #endif
 
-khm_int32 KHMAPI 
-k5_msg_ident(khm_int32 msg_type, 
-               khm_int32 msg_subtype, 
-               khm_ui_4 uparam, 
+khm_int32 KHMAPI
+k5_msg_ident(khm_int32 msg_type,
+               khm_int32 msg_subtype,
+               khm_ui_4 uparam,
                void * vparam)
 {
     switch(msg_subtype) {
@@ -1300,10 +1519,10 @@ k5_msg_ident(khm_int32 msg_type,
                              vparam);
 
     case KMSG_IDENT_VALIDATE_NAME:
-        return k5_ident_valiate_name(msg_type,
-                                     msg_subtype,
-                                     uparam,
-                                     vparam);
+        return k5_ident_validate_name(msg_type,
+                                      msg_subtype,
+                                      uparam,
+                                      vparam);
 
     case KMSG_IDENT_VALIDATE_IDENTITY:
         /* TODO: handle KMSG_IDENT_VALIDATE_IDENTITY */
@@ -1398,7 +1617,6 @@ HANDLE h_ccname_exit_event;
 HANDLE h_ccname_thread;
 
 DWORD WINAPI k5_ccname_monitor_thread(LPVOID lpParameter) {
-    krb5_context ctx = 0;
 
     HKEY hk_ccname;
     HANDLE h_notify;
@@ -1411,6 +1629,8 @@ DWORD WINAPI k5_ccname_monitor_thread(LPVOID lpParameter) {
     wchar_t reg_ccname[KRB5_MAXCCH_CCNAME];
     LONG l;
 
+    PDESCTHREAD(L"Krb5 CCName Monitor", L"Krb5");
+
     l = RegOpenKeyEx(HKEY_CURRENT_USER,
                      L"Software\\MIT\\kerberos5",
                      0,
@@ -1434,7 +1654,7 @@ DWORD WINAPI k5_ccname_monitor_thread(LPVOID lpParameter) {
     }
 
     dwSize = sizeof(reg_ccname);
-    
+
     l = RegQueryValueEx(hk_ccname,
                         L"ccname",
                         NULL,
@@ -1448,11 +1668,6 @@ DWORD WINAPI k5_ccname_monitor_thread(LPVOID lpParameter) {
         reg_ccname[0] = L'\0';
     }
 
-    l = pkrb5_init_context(&ctx);
-
-    if (l)
-        goto _exit_0;
-
     h_notify = CreateEvent(NULL, FALSE, FALSE, L"Local\\Krb5CCNameChangeNotifier");
 
     if (h_notify == NULL)
@@ -1486,7 +1701,7 @@ DWORD WINAPI k5_ccname_monitor_thread(LPVOID lpParameter) {
             wchar_t new_ccname[KRB5_MAXCCH_CCNAME];
 
             dwSize = sizeof(new_ccname);
-    
+
             l = RegQueryValueEx(hk_ccname,
                                 L"ccname",
                                 NULL,
@@ -1500,8 +1715,16 @@ DWORD WINAPI k5_ccname_monitor_thread(LPVOID lpParameter) {
             }
 
             if (_wcsicmp(new_ccname, reg_ccname)) {
-                k5_refresh_default_identity(ctx);
-                StringCbCopy(reg_ccname, sizeof(reg_ccname), new_ccname);
+                krb5_context ctx = NULL;
+
+                l = pkrb5_init_context(&ctx);
+                if (l == 0 && ctx != NULL) {
+                    k5_refresh_default_identity(ctx);
+                    StringCbCopy(reg_ccname, sizeof(reg_ccname), new_ccname);
+                }
+
+                if (ctx)
+                    pkrb5_free_context(ctx);
             }
 
         } else {
@@ -1518,14 +1741,8 @@ DWORD WINAPI k5_ccname_monitor_thread(LPVOID lpParameter) {
 
     RegCloseKey(hk_ccname);
 
-    if (ctx)
-        pkrb5_free_context(ctx);
-
  _exit:
     ExitThread(rv);
-
-    /* not reached */
-    return rv;
 }
 
 khm_int32
@@ -1539,7 +1756,7 @@ k5_msg_system_idpro(khm_int32 msg_type, khm_int32 msg_subtype,
             pkrb5_init_context(&k5_identpro_ctx);
             kcdb_identity_set_type(credtype_id_krb5);
 
-            if (KHM_FAILED(kcdb_type_get_id(TYPENAME_KRB5_PRINC, 
+            if (KHM_FAILED(kcdb_type_get_id(TYPENAME_KRB5_PRINC,
                                             &type_id_krb5_princ))) {
                 kcdb_type dt;
                 kcdb_type * pstr;
