@@ -51,8 +51,8 @@ krb5_context k5_identpro_ctx = NULL;
 /*  The system message handler.
 
     Runs in the context of the plugin thread */
-khm_int32 KHMAPI 
-k5_msg_system(khm_int32 msg_type, khm_int32 msg_subtype, 
+khm_int32 KHMAPI
+k5_msg_system(khm_int32 msg_type, khm_int32 msg_subtype,
               khm_ui_4 uparam, void * vparam)
 {
     khm_int32 rv = KHM_ERROR_SUCCESS;
@@ -70,7 +70,7 @@ k5_msg_system(khm_int32 msg_type, khm_int32 msg_subtype,
             ct.id = KCDB_CREDTYPE_AUTO;
             ct.name = KRB5_CREDTYPE_NAME;
 
-            if(LoadString(hResModule, IDS_KRB5_SHORT_DESC, 
+            if(LoadString(hResModule, IDS_KRB5_SHORT_DESC,
                           buf, ARRAYLENGTH(buf))) {
                 StringCbLength(buf, KCDB_MAXCB_SHORT_DESC, &cbsize);
                 cbsize += sizeof(wchar_t);
@@ -81,7 +81,7 @@ k5_msg_system(khm_int32 msg_type, khm_int32 msg_subtype,
             /* even though ideally we should be setting limits
                based KCDB_MAXCB_LONG_DESC, our long description
                actually fits nicely in KCDB_MAXCB_SHORT_DESC */
-            if(LoadString(hResModule, IDS_KRB5_LONG_DESC, 
+            if(LoadString(hResModule, IDS_KRB5_LONG_DESC,
                           buf, ARRAYLENGTH(buf))) {
                 StringCbLength(buf, KCDB_MAXCB_SHORT_DESC, &cbsize);
                 cbsize += sizeof(wchar_t);
@@ -111,9 +111,6 @@ k5_msg_system(khm_int32 msg_type, khm_int32 msg_subtype,
 
                 krb5_initialized = TRUE;
 
-                if(ctx != NULL)
-                    pkrb5_free_context(ctx);
-
                 /* now convert this thread to a fiber and create a
                    separate fiber to do kinit stuff */
                 k5_main_fiber = ConvertThreadToFiber(NULL);
@@ -126,6 +123,9 @@ k5_msg_system(khm_int32 msg_type, khm_int32 msg_subtype,
                 k5_register_config_panels();
 
                 khm_krb5_list_tickets(&ctx);
+
+                if(ctx != NULL)
+                    pkrb5_free_context(ctx);
             }
         }
         break;
@@ -169,13 +169,30 @@ k5_msg_system(khm_int32 msg_type, khm_int32 msg_subtype,
     return rv;
 }
 
+khm_int32 KHMAPI
+k5_msg_kcdb(khm_int32 msg_type, khm_int32 msg_subtype,
+            khm_ui_4 uparam, void * vparam)
+{
+    khm_int32 rv = KHM_ERROR_SUCCESS;
+
+    switch(msg_subtype) {
+    case KMSG_KCDB_IDENT:
+        if (uparam == KCDB_OP_DELCONFIG) {
+            k5_remove_from_LRU((khm_handle) vparam);
+        }
+        break;
+    }
+
+    return rv;
+}
+
 
 /* Handler for CRED type messages
 
     Runs in the context of the Krb5 plugin
 */
-khm_int32 KHMAPI 
-k5_msg_cred(khm_int32 msg_type, khm_int32 msg_subtype, 
+khm_int32 KHMAPI
+k5_msg_cred(khm_int32 msg_type, khm_int32 msg_subtype,
             khm_ui_4 uparam, void * vparam)
 {
     khm_int32 rv = KHM_ERROR_SUCCESS;
@@ -220,7 +237,7 @@ k5_msg_cred(khm_int32 msg_type, khm_int32 msg_subtype,
 
     default:
         if(IS_CRED_ACQ_MSG(msg_subtype))
-            return k5_msg_cred_dialog(msg_type, msg_subtype, 
+            return k5_msg_cred_dialog(msg_type, msg_subtype,
                                       uparam, vparam);
     }
 
@@ -232,8 +249,8 @@ k5_msg_cred(khm_int32 msg_type, khm_int32 msg_subtype,
 
     Runs in the context of the Krb5 plugin
 */
-khm_int32 KHMAPI 
-k5_msg_callback(khm_int32 msg_type, khm_int32 msg_subtype, 
+khm_int32 KHMAPI
+k5_msg_callback(khm_int32 msg_type, khm_int32 msg_subtype,
                 khm_ui_4 uparam, void * vparam)
 {
     switch(msg_type) {
@@ -241,6 +258,8 @@ k5_msg_callback(khm_int32 msg_type, khm_int32 msg_subtype,
         return k5_msg_system(msg_type, msg_subtype, uparam, vparam);
     case KMSG_CRED:
         return k5_msg_cred(msg_type, msg_subtype, uparam, vparam);
+    case KMSG_KCDB:
+        return k5_msg_kcdb(msg_type, msg_subtype, uparam, vparam);
     }
     return KHM_ERROR_SUCCESS;
 }
