@@ -26,6 +26,9 @@
 
 #include<khmapp.h>
 #include<assert.h>
+#if _WIN32_WINNT >= 0x0501
+#include<uxtheme.h>
+#endif
 
 static khui_config_node
 get_window_node(HWND hwnd) {
@@ -43,7 +46,7 @@ set_window_node(HWND hwnd, khui_config_node node) {
 }
 
 static void
-add_subpanels(HWND hwnd, 
+add_subpanels(HWND hwnd,
               khui_config_node ctx_node,
               khui_config_node ref_node) {
 
@@ -93,6 +96,9 @@ add_subpanels(HWND hwnd,
 #ifdef DEBUG
         assert(hwnd_panel);
 #endif
+#if _WIN32_WINNT >= 0x0501
+        EnableThemeDialogTexture(hwnd_panel, ETDT_ENABLETAB);
+#endif
 
         ShowWindow(hwnd_panel, SW_HIDE);
 
@@ -122,7 +128,7 @@ add_subpanels(HWND hwnd,
 }
 
 static void
-apply_all(HWND hwnd, 
+apply_all(HWND hwnd,
           HWND hw_tab,
           khui_config_node noderef) {
     TCITEM tci;
@@ -230,6 +236,8 @@ handle_cfg_notify(HWND hwnd,
     HWND hw;
 
     node = get_window_node(hwnd);
+    if (node == NULL)
+        return TRUE;
 
     if (HIWORD(wParam) == WMCFG_APPLY) {
 
@@ -255,13 +263,15 @@ handle_notify(HWND hwnd,
 
     lpnm = (LPNMHDR) lParam;
     node = get_window_node(hwnd);
+    if (node == NULL)
+        return FALSE;
 
     if (lpnm->idFrom == IDC_CFG_TAB) {
         switch(lpnm->code) {
         case TCN_SELCHANGING:
             i = TabCtrl_GetCurSel(lpnm->hwndFrom);
 
-            show_tab_panel(hwnd, 
+            show_tab_panel(hwnd,
                            node,
                            lpnm->hwndFrom,
                            i,
@@ -295,8 +305,9 @@ typedef struct tag_ident_data {
     wchar_t * idname;
     int lv_idx;
 
-    BOOL removed;
+    BOOL removed;               /* this identity was marked for removal */
     BOOL applied;
+    BOOL purged;                /* this identity was actually removed */
 
     khm_int32 flags;
 
@@ -452,6 +463,8 @@ write_params_ident(ident_data * d) {
         assert(!(flags & KCDB_IDENT_FLAG_CONFIG));
 #endif
 
+        d->purged = TRUE;
+
     } else {
 
         if (d->saved.monitor != d->work.monitor)
@@ -474,7 +487,7 @@ write_params_ident(ident_data * d) {
 
     d->applied = TRUE;
 
-    if (d->hwnd)
+    if (d->hwnd && !d->removed)
         PostMessage(d->hwnd, KHUI_WM_CFG_NOTIFY,
                     MAKEWPARAM(0, WMCFG_UPDATE_STATE), 0);
 
@@ -490,19 +503,19 @@ write_params_idents(void) {
         if (cfg_idents.work.monitor != cfg_idents.saved.monitor) {
             khc_write_int32(csp_cw, L"DefaultMonitor",
                             !!cfg_idents.work.monitor);
-            cfg_idents.work.monitor = cfg_idents.saved.monitor;
+            cfg_idents.saved.monitor = cfg_idents.work.monitor;
             cfg_idents.applied = TRUE;
         }
         if (cfg_idents.work.auto_renew != cfg_idents.saved.auto_renew) {
             khc_write_int32(csp_cw, L"DefaultAllowAutoRenew",
                             !!cfg_idents.work.auto_renew);
-            cfg_idents.work.auto_renew = cfg_idents.saved.auto_renew;
+            cfg_idents.saved.auto_renew = cfg_idents.work.auto_renew;
             cfg_idents.applied = TRUE;
         }
         if (cfg_idents.work.sticky != cfg_idents.saved.sticky) {
             khc_write_int32(csp_cw, L"DefaultSticky",
                             !!cfg_idents.work.sticky);
-            cfg_idents.work.sticky = cfg_idents.saved.sticky;
+            cfg_idents.saved.sticky = cfg_idents.work.sticky;
             cfg_idents.applied = TRUE;
         }
 
@@ -606,12 +619,12 @@ init_idents_data(void) {
         goto _cleanup;
     }
 
-    cfg_idents.idents = PMALLOC(sizeof(*cfg_idents.idents) * 
+    cfg_idents.idents = PMALLOC(sizeof(*cfg_idents.idents) *
                                cfg_idents.n_idents);
 #ifdef DEBUG
     assert(cfg_idents.idents);
 #endif
-    ZeroMemory(cfg_idents.idents, 
+    ZeroMemory(cfg_idents.idents,
                sizeof(*cfg_idents.idents) * cfg_idents.n_idents);
     cfg_idents.nc_idents = cfg_idents.n_idents;
 
@@ -864,11 +877,14 @@ khm_cfg_add_ident_proc(HWND hwnd,
 
         khui_cw_destroy_cred_blob(d->nc);
         PFREE(d);
+        SetWindowLongPtr(hwnd, DWLP_USER, 0);
         break;
 
     case KHUI_WM_NC_NOTIFY:
         d = (add_ident_data *)(LONG_PTR)
             GetWindowLongPtr(hwnd, DWLP_USER);
+        if (d == NULL)
+            break;
 
         switch(HIWORD(wParam)) {
         case WMNC_ADD_CONTROL_ROW:
@@ -958,6 +974,7 @@ khm_cfg_add_ident_proc(HWND hwnd,
             khm_handle csp_ident = NULL;
             khm_size cb;
             khm_int32 rv = KHM_ERROR_SUCCESS;
+            khm_int32 flags = 0;
 
             d = (add_ident_data *)(LONG_PTR)
                 GetWindowLongPtr(hwnd, DWLP_USER);
@@ -965,6 +982,9 @@ khm_cfg_add_ident_proc(HWND hwnd,
             if (!d || !d->nc)
                 break;
 
+            if (d->nc->ident_cb)
+                d->nc->ident_cb(d->nc, WMNC_IDENT_PREPROCESS, NULL, 0, 0, 0);
+
             /* check if there was an identity selected */
             if (d->nc->n_identities == 0 ||
                 d->nc->identities[0] == NULL) {
@@ -983,6 +1003,23 @@ khm_cfg_add_ident_proc(HWND hwnd,
             cb = sizeof(idname);
             kcdb_identity_get_name(ident, idname, &cb);
 
+            /* check if the identity is already in the
+               configuration */
+            if (KHM_SUCCEEDED(kcdb_identity_get_flags(ident, &flags)) &&
+                (flags & KCDB_IDENT_FLAG_CONFIG)) {
+
+                wchar_t fmt[256];
+
+                LoadString(khm_hInstance, IDS_CFG_IDNAME_EXT,
+                           fmt, ARRAYLENGTH(fmt));
+                StringCbPrintf(err_msg, sizeof(err_msg), fmt, idname);
+
+                kcdb_identity_release(ident);
+                ident = NULL;
+
+                goto show_failure;
+            }
+
             /* now we have to create the identity configuration. */
             if (KHM_FAILED(rv = kcdb_identity_get_config(ident,
                                                          KHM_FLAG_CREATE,
@@ -994,6 +1031,7 @@ khm_cfg_add_ident_proc(HWND hwnd,
                 StringCbPrintf(err_msg, sizeof(err_msg), fmt, rv);
 
                 kcdb_identity_release(ident);
+                ident = NULL;
 
                 goto show_failure;
             }
@@ -1029,7 +1067,7 @@ khm_cfg_add_ident_proc(HWND hwnd,
                 break;
             }
             break;
-            
+
         } else if (LOWORD(wParam) == IDCANCEL) {
             EndDialog(hwnd, 1);
         } else {
@@ -1069,9 +1107,9 @@ khm_cfg_ids_tab_proc(HWND hwnd,
             if (cfg_idents.hi_status)
                 goto _done_with_icons;
 
-            cfg_idents.hi_status = 
+            cfg_idents.hi_status =
                 ImageList_Create(GetSystemMetrics(SM_CXSMICON),
-                                 GetSystemMetrics(SM_CYSMICON), 
+                                 GetSystemMetrics(SM_CYSMICON),
                                  ILC_COLOR8 | ILC_MASK,
                                  4,4);
 
@@ -1087,37 +1125,37 @@ khm_cfg_ids_tab_proc(HWND hwnd,
             DestroyIcon(hicon);
 
             hicon = LoadImage(khm_hInstance, MAKEINTRESOURCE(IDI_CFG_DEFAULT),
-                              IMAGE_ICON, GetSystemMetrics(SM_CXSMICON), 
+                              IMAGE_ICON, GetSystemMetrics(SM_CXSMICON),
                               GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
 
-            cfg_idents.idx_default = ImageList_AddIcon(cfg_idents.hi_status, 
+            cfg_idents.idx_default = ImageList_AddIcon(cfg_idents.hi_status,
                                                        hicon) + 1;
 
             DestroyIcon(hicon);
 
             hicon = LoadImage(khm_hInstance, MAKEINTRESOURCE(IDI_CFG_MODIFIED),
-                              IMAGE_ICON, GetSystemMetrics(SM_CXSMICON), 
+                              IMAGE_ICON, GetSystemMetrics(SM_CXSMICON),
                               GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
 
-            cfg_idents.idx_modified = ImageList_AddIcon(cfg_idents.hi_status, 
+            cfg_idents.idx_modified = ImageList_AddIcon(cfg_idents.hi_status,
                                                         hicon) + 1;
 
             DestroyIcon(hicon);
 
             hicon = LoadImage(khm_hInstance, MAKEINTRESOURCE(IDI_CFG_APPLIED),
-                              IMAGE_ICON, GetSystemMetrics(SM_CXSMICON), 
+                              IMAGE_ICON, GetSystemMetrics(SM_CXSMICON),
                               GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
 
-            cfg_idents.idx_applied = ImageList_AddIcon(cfg_idents.hi_status, 
+            cfg_idents.idx_applied = ImageList_AddIcon(cfg_idents.hi_status,
                                                        hicon) + 1;
 
             DestroyIcon(hicon);
 
             hicon = LoadImage(khm_hInstance, MAKEINTRESOURCE(IDI_CFG_DELETED),
-                              IMAGE_ICON, GetSystemMetrics(SM_CXSMICON), 
+                              IMAGE_ICON, GetSystemMetrics(SM_CXSMICON),
                               GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
 
-            cfg_idents.idx_deleted = ImageList_AddIcon(cfg_idents.hi_status, 
+            cfg_idents.idx_deleted = ImageList_AddIcon(cfg_idents.hi_status,
                                                        hicon) + 1;
 
             DestroyIcon(hicon);
@@ -1240,8 +1278,17 @@ find_ident_by_node(khui_config_node node) {
             break;
     }
 
-    if (i < (int)cfg_idents.n_idents)
+    if (i < (int)cfg_idents.n_idents) {
+        if (cfg_idents.idents[i].purged) {
+            /* we are re-creating a purged identity */
+            cfg_idents.idents[i].purged = FALSE;
+            cfg_idents.idents[i].removed = FALSE;
+            cfg_idents.idents[i].applied = FALSE;
+
+            read_params_ident(&cfg_idents.idents[i]);
+        }
         return &cfg_idents.idents[i];
+    }
 
     /* there is no identity data for this configuration node.  We try
        to create it. */
@@ -1456,11 +1503,14 @@ khm_cfg_id_tab_proc(HWND hwnd,
                 cont = (BOOL *) lParam;
                 d = find_ident_by_node(idata->ctx_node);
                 write_params_ident(d);
-                if (d->removed && cont)
-                    *cont = FALSE;
-                khui_cfg_set_flags_inst(idata, KHUI_CNFLAG_APPLIED,
-                                        KHUI_CNFLAG_APPLIED | 
-                                        KHUI_CNFLAG_MODIFIED);
+                if (d->removed) {
+                    if (cont)
+                        *cont = FALSE;
+                } else {
+                    khui_cfg_set_flags_inst(idata, KHUI_CNFLAG_APPLIED,
+                                            KHUI_CNFLAG_APPLIED |
+                                            KHUI_CNFLAG_MODIFIED);
+                }
                 break;
 
             case WMCFG_UPDATE_STATE:
