@@ -45,7 +45,7 @@ typedef struct tag_cfgui_wnd_data {
 
 static cfgui_wnd_data *
 cfgui_get_wnd_data(HWND hwnd) {
-    return (cfgui_wnd_data *)(LONG_PTR) 
+    return (cfgui_wnd_data *)(LONG_PTR)
         GetWindowLongPtr(hwnd, DWLP_USER);
 }
 
@@ -132,7 +132,7 @@ cfgui_add_node(cfgui_wnd_data * d,
     }
 }
 
-static void 
+static void
 cfgui_initialize_dialog(HWND hwnd) {
     cfgui_wnd_data * d;
     HWND hwtv;
@@ -142,10 +142,12 @@ cfgui_initialize_dialog(HWND hwnd) {
     HICON hicon;
 
     d = cfgui_get_wnd_data(hwnd);
+    if (d == NULL)
+        return;
 
     /* create and fill the image list for the treeview */
 
-    d->hi_status = ImageList_Create(GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), 
+    d->hi_status = ImageList_Create(GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON),
                                     ILC_COLOR8 | ILC_MASK,
                                     4,4);
 
@@ -182,8 +184,8 @@ cfgui_initialize_dialog(HWND hwnd) {
     cfgui_add_node(d, hwtv, NULL, NULL, FALSE);
 
     hdc = GetDC(hwnd);
-    hf = CreateFont(-MulDiv(12, 
-                            GetDeviceCaps(hdc, LOGPIXELSY), 
+    hf = CreateFont(-MulDiv(12,
+                            GetDeviceCaps(hdc, LOGPIXELSY),
                             72),
                     0,          /* nWidth */
                     0,          /* nEscapement */
@@ -241,6 +243,8 @@ cfgui_uninitialize_dialog(HWND hwnd) {
     HWND hwtv;
 
     d = cfgui_get_wnd_data(hwnd);
+    if (d == NULL)
+        return;
 
     hwtv = GetDlgItem(hwnd, IDC_CFG_NODELIST);
 
@@ -298,6 +302,9 @@ cfgui_activate_node(HWND hwnd, khui_config_node node) {
     HWND hwtv;
 
     d = cfgui_get_wnd_data(hwnd);
+    if (d == NULL)
+        return;
+
     hwtv = GetDlgItem(hwnd, IDC_CFG_NODELIST);
     hItem = (HTREEITEM) khui_cfg_get_param(node);
 
@@ -423,7 +430,210 @@ cfgui_apply_settings(khui_config_node node) {
 }
 
 static void
-cfgui_update_state(HWND hwnd, 
+cfgui_remove_item(HWND hwtv,
+                  HTREEITEM hItem) {
+    khui_config_node node;
+    HTREEITEM hChild;
+    TVITEMEX itemex;
+
+    for (hChild = TreeView_GetChild(hwtv, hItem);
+         hChild;
+         hChild = TreeView_GetChild(hwtv, hItem)) {
+
+        cfgui_remove_item(hwtv, hChild);
+
+    }
+
+    ZeroMemory(&itemex, sizeof(itemex));
+
+    itemex.mask = TVIF_PARAM;
+    itemex.hItem = hItem;
+
+    TreeView_GetChild(hwtv, &itemex);
+
+    node = (khui_config_node) itemex.lParam;
+
+    if (node) {
+        HWND hw;
+        hw = khui_cfg_get_hwnd(node);
+
+        if (hw)
+            DestroyWindow(hw);
+
+        khui_cfg_release(node);
+    }
+
+    TreeView_DeleteItem(hwtv, hItem);
+}
+
+struct cfgui_child_info {
+    HTREEITEM hItem;
+    khui_config_node node;
+    BOOL checked;
+};
+
+#define CI_ALLOC_INCR 8
+
+static void
+cfgui_sync_node(cfgui_wnd_data * d,
+                HWND hwtv,
+                khui_config_node c,
+                HTREEITEM hItem) {
+    khui_config_node child;
+    HTREEITEM hChild;
+    struct cfgui_child_info * childinfo = NULL;
+    khm_size n_childinfo = 0;
+    khm_size nc_childinfo = 0;
+    khm_size i;
+
+    /* first, get the list of children from the treeview control */
+    for (hChild = TreeView_GetChild(hwtv, hItem);
+         hChild;
+         hChild = TreeView_GetNextSibling(hwtv, hChild)) {
+
+        if (n_childinfo >= nc_childinfo) {
+            nc_childinfo = UBOUNDSS(n_childinfo + 1,
+                                    CI_ALLOC_INCR, CI_ALLOC_INCR);
+#ifdef DEBUG
+            assert(nc_childinfo > n_childinfo);
+#endif
+            childinfo = PREALLOC(childinfo,
+                                 sizeof(*childinfo) * nc_childinfo);
+#ifdef DEBUG
+            assert(childinfo);
+#endif
+        }
+
+        ZeroMemory(&childinfo[n_childinfo],
+                   sizeof(childinfo[n_childinfo]));
+
+        childinfo[n_childinfo].hItem = hChild;
+        childinfo[n_childinfo].checked = FALSE;
+        n_childinfo++;
+    }
+
+    /* now, go through the list of actual nodes and make sure they
+       match up */
+    child = NULL;
+    for (khui_cfg_get_first_child(c, &child);
+         child;
+         khui_cfg_get_next_release(&child)) {
+
+        hChild = (HTREEITEM) khui_cfg_get_param(child);
+
+        for (i=0; i < n_childinfo; i++) {
+            if (childinfo[i].hItem == hChild)
+                break;
+        }
+
+        if (i < n_childinfo) {
+            childinfo[i].checked = TRUE;
+        } else {
+            /* add it to the list, so we can create the node in the
+               tree view control later. */
+            if (n_childinfo >= nc_childinfo) {
+                nc_childinfo = UBOUNDSS(n_childinfo + 1,
+                                        CI_ALLOC_INCR, CI_ALLOC_INCR);
+#ifdef DEBUG
+                assert(nc_childinfo > n_childinfo);
+#endif
+                childinfo = PREALLOC(childinfo,
+                                     sizeof(*childinfo) * nc_childinfo);
+#ifdef DEBUG
+                assert(childinfo);
+#endif
+            }
+
+            ZeroMemory(&childinfo[n_childinfo],
+                       sizeof(childinfo[n_childinfo]));
+
+            childinfo[n_childinfo].node = child;
+            khui_cfg_hold(child);
+            n_childinfo++;
+        }
+    }
+
+    /* by this point, the childinfo list contains items of the
+       following forms:
+
+       1. childinfo[i].hItem != NULL && childinfo[i].checked == TRUE
+
+          Corresponds to a tree view item that has a matching
+          configuration node.  Nothing to do here.
+
+       2. childinfo[i].hItem != NULL && childinfo[i].checked == FALSE
+
+          Corresponds to a tree view item that has no matching
+          configuration node.  These should be removed.
+
+       3. childinfo[i].hItem == NULL && childinfo[i].node != NULL
+
+          Corresponds to a configuration node that has no matching
+          tree view item.  These nodes should be added.
+    */
+
+    /* first do the removals */
+    for (i=0; i < n_childinfo; i++) {
+        if (childinfo[i].hItem == NULL)
+            break;              /* nothing more to see from this point
+                                   on */
+        if (!childinfo[i].checked) {
+            /* remove! */
+            cfgui_remove_item(hwtv, childinfo[i].hItem);
+        }
+    }
+
+    /* continue from where the previous loop left off */
+    for (; i < n_childinfo; i++) {
+#ifdef DEBUG
+        assert(childinfo[i].hItem == NULL);
+        assert(childinfo[i].node != NULL);
+#endif
+
+        cfgui_add_node(d, hwtv, childinfo[i].node, c, FALSE);
+
+        khui_cfg_release(childinfo[i].node);
+        childinfo[i].node = NULL;
+    }
+
+    if (childinfo)
+        PFREE(childinfo);
+
+    /* finally recurse through to the next level */
+    for (hChild = TreeView_GetChild(hwtv, hItem);
+         hChild;
+         hChild = TreeView_GetNextSibling(hwtv, hChild)) {
+
+        TVITEMEX itemex;
+
+        ZeroMemory(&itemex, sizeof(itemex));
+
+        itemex.mask = TVIF_PARAM;
+        itemex.hItem = hChild;
+
+        TreeView_GetItem(hwtv, &itemex);
+
+        if (itemex.lParam) {
+            child = (khui_config_node) itemex.lParam;
+
+            cfgui_sync_node(d, hwtv, child, hChild);
+        }
+    }
+}
+
+static void
+cfgui_sync_node_list(cfgui_wnd_data * d, HWND hwnd) {
+    HWND hwtv;
+    HTREEITEM hItem;
+
+    hwtv = GetDlgItem(hwnd, IDC_CFG_NODELIST);
+    hItem = TreeView_GetRoot(hwtv);
+
+    cfgui_sync_node(d, hwtv, NULL, hItem);
+}
+
+static void
+cfgui_update_state(HWND hwnd,
                    khm_int32 flags,
                    khui_config_node node) {
     cfgui_wnd_data * d;
@@ -433,6 +643,9 @@ cfgui_update_state(HWND hwnd,
     int idx;
 
     d = cfgui_get_wnd_data(hwnd);
+    if (d == NULL)
+        return;
+
     hwtv = GetDlgItem(hwnd, IDC_CFG_NODELIST);
     hItem = (HTREEITEM) khui_cfg_get_param(node);
 
@@ -476,6 +689,9 @@ cfgui_dlgproc_generic(HWND hwnd,
 
     case WM_CTLCOLORSTATIC:
         d = cfgui_get_wnd_data(hwnd);
+        if (d == NULL)
+            break;
+
         return (BOOL)(DWORD_PTR) d->hbr_white;
 
     case WM_ERASEBKGND:
@@ -486,6 +702,8 @@ cfgui_dlgproc_generic(HWND hwnd,
             RECT r_fill;
 
             d = cfgui_get_wnd_data(hwnd);
+            if (d == NULL)
+                break;
 
             GetClientRect(hwnd, &r_client);
             SetRectEmpty(&r_logo);
@@ -521,7 +739,7 @@ cfgui_dlgproc_generic(HWND hwnd,
     return FALSE;
 }
 
-static INT_PTR CALLBACK 
+static INT_PTR CALLBACK
 cfgui_dlgproc(HWND hwnd,
               UINT uMsg,
               WPARAM wParam,
@@ -543,7 +761,7 @@ cfgui_dlgproc(HWND hwnd,
 
         d->hbr_white = CreateSolidBrush(RGB(255,255,255));
 
-        d->hw_generic_pane = 
+        d->hw_generic_pane =
             CreateDialogParam(khm_hInstance,
                               MAKEINTRESOURCE(IDD_CFG_GENERIC),
                               hwnd,
@@ -578,13 +796,20 @@ cfgui_dlgproc(HWND hwnd,
         cfgui_uninitialize_dialog(hwnd);
 
         d = cfgui_get_wnd_data(hwnd);
+        if (d == NULL)
+            break;
+
         khui_delete_bitmap(&d->kbmp_logo);
         DeleteObject(d->hbr_white);
 
+        cfgui_set_wnd_data(hwnd, NULL);
+
         khm_del_dialog(hwnd);
 
         SetForegroundWindow(khm_hwnd_main);
 
+        PFREE(d);
+
         return FALSE;
 
     case WM_NOTIFY:
@@ -600,7 +825,7 @@ cfgui_dlgproc(HWND hwnd,
             case TVN_SELCHANGED:
                 lptv = (LPNMTREEVIEW) lParam;
                 cfgui_activate_node(hwnd,
-                                    (khui_config_node) 
+                                    (khui_config_node)
                                     lptv->itemNew.lParam);
                 return TRUE;
 
@@ -635,6 +860,9 @@ cfgui_dlgproc(HWND hwnd,
     case WM_CTLCOLORSTATIC:
         {
             d = cfgui_get_wnd_data(hwnd);
+            if (d == NULL)
+                break;
+
             return (BOOL)(DWORD_PTR) d->hbr_white;
         }
         /* implicit break */
@@ -665,12 +893,16 @@ cfgui_dlgproc(HWND hwnd,
             break;
 
         case WMCFG_UPDATE_STATE:
-            cfgui_update_state(hwnd, LOWORD(wParam), 
+            cfgui_update_state(hwnd, LOWORD(wParam),
                                (khui_config_node) lParam);
             break;
 
         case WMCFG_SYNC_NODE_LIST:
-            /*TODO: synchronize the node lists here */
+            d = cfgui_get_wnd_data(hwnd);
+            if (d == NULL)
+                break;
+
+            cfgui_sync_node_list(d, hwnd);
             break;
         }
 
@@ -680,7 +912,7 @@ cfgui_dlgproc(HWND hwnd,
     return FALSE;
 }
 
-static void 
+static void
 cfgui_create_window(khui_config_node node) {
 #ifdef DEBUG
     assert(cfgui_hwnd == NULL);
@@ -699,14 +931,14 @@ cfgui_create_window(khui_config_node node) {
     ShowWindow(cfgui_hwnd,SW_SHOW);
 }
 
-static void 
+static void
 cfgui_destroy_window(void) {
     if (cfgui_hwnd)
         DestroyWindow(cfgui_hwnd);
     /* cfgui_hwnd will be set to NULL in the dialog proc */
 }
 
-void 
+void
 khm_show_config_pane(khui_config_node node) {
     if (cfgui_hwnd != NULL) {
         SendMessage(cfgui_hwnd, KHUI_WM_CFG_NOTIFY,
@@ -825,6 +1057,9 @@ void khm_refresh_config(void) {
 
             khui_cfg_remove(cfg_iter);
         }
+
+        if (tident)
+            kcdb_identity_release(tident);
     }
 
     /* Now iterate through the root level configuration nodes and make
@@ -836,6 +1071,8 @@ void khm_refresh_config(void) {
     if (omenu == NULL)
         goto _cleanup;
 
+    khui_action_lock();
+
     do {
         khm_int32 action;
         khm_int32 flags;
@@ -882,7 +1119,7 @@ void khm_refresh_config(void) {
                 goto _next_cfg;
             }
 
-            khui_menu_insert_action(omenu, -1, action, 0);
+            khui_menu_insert_action(omenu, (khm_size) -1, action, 0);
 
             refresh_menu = TRUE;
         }
@@ -892,8 +1129,11 @@ void khm_refresh_config(void) {
             break;
     } while(cfg_r);
 
-    if (refresh_menu)
-        khm_menu_refresh_items();
+    khui_action_unlock();
+
+    if (refresh_menu) {
+        khui_refresh_actions();
+    }
 
  _cleanup:
     if (cfg_ids)
