@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2005 Massachusetts Institute of Technology
+ * Copyright (c) 2007 Secure Endpoints Inc.
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -33,6 +34,7 @@ static HANDLE in_dialog_evt = NULL;
 static LONG init_dialog = 0;
 static khm_int32 dialog_result = 0;
 static wchar_t dialog_identity[KCDB_IDENT_MAXCCH_NAME];
+static khui_new_creds * dialog_nc = NULL;
 
 static void
 dialog_sync_init(void) {
@@ -56,7 +58,7 @@ dialog_sync_init(void) {
     }
 }
 
-BOOL 
+BOOL
 khm_cred_begin_dialog(void) {
     BOOL rv;
 
@@ -64,9 +66,28 @@ khm_cred_begin_dialog(void) {
 
     EnterCriticalSection(&cs_dialog);
 
-    if (in_dialog)
+    if (in_dialog) {
         rv = FALSE;
-    else {
+
+        /* if a dialog is being displayed and we got a another request
+           to show one, we bring the existing one to the
+           foreground. */
+        if (dialog_nc && dialog_nc->hwnd) {
+            khm_int32 t = 0;
+
+            if (KHM_SUCCEEDED(khc_read_int32(NULL,
+                                             L"CredWindow\\Windows\\NewCred\\ForceToTop",
+                                             &t)) &&
+                t != 0) {
+
+                khm_activate_main_window();
+
+                SetWindowPos(dialog_nc->hwnd, HWND_TOP, 0, 0, 0, 0,
+                             (SWP_NOMOVE | SWP_NOSIZE));
+            }
+        }
+
+    } else {
         rv = TRUE;
         in_dialog = TRUE;
         ResetEvent(in_dialog_evt);
@@ -76,7 +97,7 @@ khm_cred_begin_dialog(void) {
     return rv;
 }
 
-void 
+void
 khm_cred_end_dialog(khui_new_creds * nc) {
     dialog_sync_init();
 
@@ -86,6 +107,10 @@ khm_cred_end_dialog(khui_new_creds * nc) {
         SetEvent(in_dialog_evt);
     }
     dialog_result = nc->result;
+#ifdef DEBUG
+    assert(dialog_nc == nc);
+#endif
+    dialog_nc = NULL;
     if (nc->subtype == KMSG_CRED_NEW_CREDS &&
         nc->n_identities > 0 &&
         nc->identities[0]) {
@@ -155,8 +180,12 @@ khm_cred_wait_for_dialog(DWORD timeout, khm_int32 * result,
     return rv;
 }
 
-/* completion handler for KMSG_CRED messages */
-void KHMAPI 
+/* Completion handler for KMSG_CRED messages.  We control the overall
+   logic of credentials acquisition and other operations here.  Once a
+   credentials operation is triggered, each successive message
+   completion notification will be used to dispatch the messages for
+   the next step in processing the operation. */
+void KHMAPI
 kmsg_cred_completion(kmq_message *m)
 {
     khui_new_creds * nc;
@@ -174,7 +203,7 @@ kmsg_cred_completion(kmq_message *m)
     case KMSG_CRED_NEW_CREDS:
         /* Cred types have attached themselves.  Trigger the next
            phase. */
-        kmq_post_message(KMSG_CRED, KMSG_CRED_DIALOG_SETUP, 0, 
+        kmq_post_message(KMSG_CRED, KMSG_CRED_DIALOG_SETUP, 0,
                          m->vparam);
         break;
 
@@ -182,7 +211,7 @@ kmsg_cred_completion(kmq_message *m)
         nc = (khui_new_creds *) m->vparam;
 
         /* khm_cred_dispatch_process_message() deals with the case
-           where there are not credential types that wants to
+           where there are no credential types that wants to
            participate in this operation. */
         khm_cred_dispatch_process_message(nc);
         break;
@@ -191,17 +220,17 @@ kmsg_cred_completion(kmq_message *m)
         nc = (khui_new_creds *) m->vparam;
 
         khm_prep_newcredwnd(nc->hwnd);
-            
+
         /* all the controls have been created.  Now initialize them */
         if (nc->n_types > 0) {
-            kmq_post_subs_msg(nc->type_subs, 
-                              nc->n_types, 
-                              KMSG_CRED, 
-                              KMSG_CRED_DIALOG_PRESTART, 
-                              0, 
+            kmq_post_subs_msg(nc->type_subs,
+                              nc->n_types,
+                              KMSG_CRED,
+                              KMSG_CRED_DIALOG_PRESTART,
+                              0,
                               m->vparam);
         } else {
-            PostMessage(nc->hwnd, KHUI_WM_NC_NOTIFY, 
+            PostMessage(nc->hwnd, KHUI_WM_NC_NOTIFY,
                         MAKEWPARAM(0, WMNC_DIALOG_PROCESS_COMPLETE), 0);
         }
         break;
@@ -210,12 +239,12 @@ kmsg_cred_completion(kmq_message *m)
         /* all prestart stuff is done.  Now to activate the dialog */
         nc = (khui_new_creds *) m->vparam;
         khm_show_newcredwnd(nc->hwnd);
-        
+
         kmq_post_subs_msg(nc->type_subs,
                           nc->n_types,
-                          KMSG_CRED, 
-                          KMSG_CRED_DIALOG_START, 
-                          0, 
+                          KMSG_CRED,
+                          KMSG_CRED_DIALOG_START,
+                          0,
                           m->vparam);
         /* at this point, the dialog window takes over.  We let it run
            the show until KMSG_CRED_DIALOG_END is posted by the dialog
@@ -227,6 +256,9 @@ kmsg_cred_completion(kmq_message *m)
            if there's more */
         nc = (khui_new_creds *) m->vparam;
 
+        /* if we are done processing all the plug-ins, then check if
+           there were any errors reported.  Otherwise we dispatch
+           another set of messages. */
         if(!khm_cred_dispatch_process_level(nc)) {
 
             if(kherr_is_error()) {
@@ -238,42 +270,143 @@ kmsg_cred_completion(kmq_message *m)
                 wchar_t ws_title[ARRAYLENGTH(ws_tfmt) + KCDB_IDENT_MAXCCH_NAME];
                 khm_size cb;
 
+                /* For renewals, we suppress the error message for the
+                   following case:
+
+                   - The renewal was for an identity
+
+                   - There are no identity credentials for the
+                     identity (no credentials that have the same type
+                     as the identity provider). */
+
+                if (nc->subtype == KMSG_CRED_RENEW_CREDS &&
+                    nc->ctx.scope == KHUI_SCOPE_IDENT &&
+                    nc->ctx.identity != NULL) {
+                    khm_handle tcs = NULL; /* credential set */
+                    khm_size count = 0;
+                    khm_int32 id_ctype = KCDB_CREDTYPE_INVALID;
+                    khm_int32 delta = 0;
+
+                    kcdb_identity_get_type(&id_ctype);
+                    kcdb_credset_create(&tcs);
+                    kcdb_credset_collect(tcs, NULL,
+                                         nc->ctx.identity,
+                                         id_ctype,
+                                         &delta);
+                    kcdb_credset_get_size(tcs, &count);
+                    kcdb_credset_delete(tcs);
+
+                    if (count == 0) {
+                        goto done_with_op;
+                    }
+                }
+
                 ctx = kherr_peek_context();
                 evt = kherr_get_err_event(ctx);
                 kherr_evaluate_event(evt);
 
                 khui_alert_create_empty(&alert);
 
-                if (nc->subtype == KMSG_CRED_PASSWORD)
-                    LoadString(khm_hInstance, IDS_NC_PWD_FAILED_TITLE,
-                               ws_tfmt, ARRAYLENGTH(ws_tfmt));
-                else if (nc->subtype == KMSG_CRED_RENEW_CREDS)
-                    LoadString(khm_hInstance, IDS_NC_REN_FAILED_TITLE,
-                               ws_tfmt, ARRAYLENGTH(ws_tfmt));
-                else
-                    LoadString(khm_hInstance, IDS_NC_FAILED_TITLE,
-                               ws_tfmt, ARRAYLENGTH(ws_tfmt));
+                if (nc->subtype == KMSG_CRED_NEW_CREDS) {
+
+                    khui_alert_set_type(alert, KHUI_ALERTTYPE_ACQUIREFAIL);
 
-                if (nc->n_identities > 0) {
                     cb = sizeof(w_idname);
-                    if (KHM_FAILED(kcdb_identity_get_name(nc->identities[0], 
-                                                          w_idname, &cb)))
-                        StringCbCopy(w_idname, sizeof(w_idname), L"(?)");
-                } else {
-                    StringCbCopy(w_idname, sizeof(w_idname), L"(?)");
-                }
+                    if (nc->n_identities == 0 ||
+                        KHM_FAILED(kcdb_identity_get_name(nc->identities[0],
+                                                          w_idname, &cb))) {
+                        /* an identity could not be determined */
+                        LoadString(khm_hInstance, IDS_NC_FAILED_TITLE,
+                                   ws_title, ARRAYLENGTH(ws_title));
+                    } else {
+                        LoadString(khm_hInstance, IDS_NC_FAILED_TITLE_I,
+                                   ws_tfmt, ARRAYLENGTH(ws_tfmt));
+                        StringCbPrintf(ws_title, sizeof(ws_title),
+                                       ws_tfmt, w_idname);
+                        khui_alert_set_ctx(alert,
+                                           KHUI_SCOPE_IDENT,
+                                           nc->identities[0],
+                                           KCDB_CREDTYPE_INVALID,
+                                           NULL);
+                    }
 
-                StringCbPrintf(ws_title, sizeof(ws_title), ws_tfmt, w_idname);
+                } else if (nc->subtype == KMSG_CRED_PASSWORD) {
+
+                    khui_alert_set_type(alert, KHUI_ALERTTYPE_CHPW);
+
+                    cb = sizeof(w_idname);
+                    if (nc->n_identities == 0 ||
+                        KHM_FAILED(kcdb_identity_get_name(nc->identities[0],
+                                                          w_idname, &cb))) {
+                        LoadString(khm_hInstance, IDS_NC_PWD_FAILED_TITLE,
+                                   ws_title, ARRAYLENGTH(ws_title));
+                    } else {
+                        LoadString(khm_hInstance, IDS_NC_PWD_FAILED_TITLE_I,
+                                   ws_tfmt, ARRAYLENGTH(ws_tfmt));
+                        StringCbPrintf(ws_title, sizeof(ws_title),
+                                       ws_tfmt, w_idname);
+                        khui_alert_set_ctx(alert,
+                                           KHUI_SCOPE_IDENT,
+                                           nc->identities[0],
+                                           KCDB_CREDTYPE_INVALID,
+                                           NULL);
+                    }
+
+                } else if (nc->subtype == KMSG_CRED_RENEW_CREDS) {
+
+                    khui_alert_set_type(alert, KHUI_ALERTTYPE_RENEWFAIL);
+
+                    cb = sizeof(w_idname);
+                    if (nc->ctx.identity == NULL ||
+                        KHM_FAILED(kcdb_identity_get_name(nc->ctx.identity,
+                                                          w_idname, &cb))) {
+                        LoadString(khm_hInstance, IDS_NC_REN_FAILED_TITLE,
+                                   ws_title, ARRAYLENGTH(ws_title));
+                    } else {
+                        LoadString(khm_hInstance, IDS_NC_REN_FAILED_TITLE_I,
+                                   ws_tfmt, ARRAYLENGTH(ws_tfmt));
+                        StringCbPrintf(ws_title, sizeof(ws_title),
+                                       ws_tfmt, w_idname);
+                        khui_alert_set_ctx(alert,
+                                           KHUI_SCOPE_IDENT,
+                                           nc->ctx.identity,
+                                           KCDB_CREDTYPE_INVALID,
+                                           NULL);
+                    }
+
+                } else {
+#ifdef DEBUG
+                    assert(FALSE);
+#endif
+                }
 
                 khui_alert_set_title(alert, ws_title);
                 khui_alert_set_severity(alert, evt->severity);
+
                 if(!evt->long_desc)
                     khui_alert_set_message(alert, evt->short_desc);
                 else
                     khui_alert_set_message(alert, evt->long_desc);
+
                 if(evt->suggestion)
                     khui_alert_set_suggestion(alert, evt->suggestion);
 
+                if (nc->subtype == KMSG_CRED_RENEW_CREDS &&
+                    nc->ctx.identity != NULL) {
+
+                    khm_int32 n_cmd;
+
+                    n_cmd = khm_get_identity_new_creds_action(nc->ctx.identity);
+
+                    if (n_cmd != 0) {
+                        khui_alert_add_command(alert, n_cmd);
+                        khui_alert_add_command(alert, KHUI_PACTION_CLOSE);
+
+                        khui_alert_set_flags(alert, KHUI_ALERT_FLAG_DISPATCH_CMD,
+                                             KHUI_ALERT_FLAG_DISPATCH_CMD);
+                    }
+                }
+
                 khui_alert_show(alert);
                 khui_alert_release(alert);
 
@@ -282,11 +415,13 @@ kmsg_cred_completion(kmq_message *m)
                 kherr_clear_error();
             }
 
+        done_with_op:
+
             if (nc->subtype == KMSG_CRED_RENEW_CREDS) {
-                kmq_post_message(KMSG_CRED, KMSG_CRED_END, 0, 
+                kmq_post_message(KMSG_CRED, KMSG_CRED_END, 0,
                                  m->vparam);
             } else {
-                PostMessage(nc->hwnd, KHUI_WM_NC_NOTIFY, 
+                PostMessage(nc->hwnd, KHUI_WM_NC_NOTIFY,
                             MAKEWPARAM(0, WMNC_DIALOG_PROCESS_COMPLETE),
                             0);
             }
@@ -297,25 +432,37 @@ kmsg_cred_completion(kmq_message *m)
         /* all is done. */
         {
             khui_new_creds * nc;
+            khm_boolean continue_cmdline = TRUE;
 
             nc = (khui_new_creds *) m->vparam;
 
             if (nc->subtype == KMSG_CRED_NEW_CREDS ||
                 nc->subtype == KMSG_CRED_PASSWORD) {
 
-                /*
-                if (nc->subtype == KMSG_CRED_NEW_CREDS)
-                    khui_context_reset();
-                */
-
                 khm_cred_end_dialog(nc);
+
+            } else if (nc->subtype == KMSG_CRED_RENEW_CREDS) {
+
+                /* if this is a renewal that was triggered while we
+                   were processing the commandline, then we need to
+                   update the pending renewal count. */
+
+                if (khm_startup.processing) {
+                    LONG renewals;
+                    renewals = InterlockedDecrement(&khm_startup.pending_renewals);
+
+                    if (renewals != 0) {
+                        continue_cmdline = FALSE;
+                    }
+                }
             }
 
             khui_cw_destroy_cred_blob(nc);
 
             kmq_post_message(KMSG_CRED, KMSG_CRED_REFRESH, 0, 0);
 
-            kmq_post_message(KMSG_ACT, KMSG_ACT_CONTINUE_CMDLINE, 0, 0);
+            if (continue_cmdline)
+                kmq_post_message(KMSG_ACT, KMSG_ACT_CONTINUE_CMDLINE, 0, 0);
         }
         break;
 
@@ -324,12 +471,12 @@ kmsg_cred_completion(kmq_message *m)
     case KMSG_CRED_PP_BEGIN:
         /* all the pages should have been added by now.  Just send out
            the precreate message */
-        kmq_post_message(KMSG_CRED, KMSG_CRED_PP_PRECREATE, 0, 
+        kmq_post_message(KMSG_CRED, KMSG_CRED_PP_PRECREATE, 0,
                          m->vparam);
         break;
 
     case KMSG_CRED_PP_END:
-        kmq_post_message(KMSG_CRED, KMSG_CRED_PP_DESTROY, 0, 
+        kmq_post_message(KMSG_CRED, KMSG_CRED_PP_DESTROY, 0,
                          m->vparam);
         break;
 
@@ -346,7 +493,35 @@ kmsg_cred_completion(kmq_message *m)
         break;
 
     case KMSG_CRED_IMPORT:
-        kmq_post_message(KMSG_ACT, KMSG_ACT_CONTINUE_CMDLINE, 0, 0);
+        {
+            khm_boolean continue_cmdline = FALSE;
+            LONG pending_renewals;
+
+            /* once an import operation ends, we have to trigger a
+               renewal so that other plug-ins that didn't participate
+               in the import operation can have a chance at getting
+               the necessary credentials.
+
+               If we are in the middle of processing the commandline,
+               we have to be a little bit careful.  We can't issue a
+               commandline conituation message right now because the
+               import action is still ongoing (since the renewals are
+               part of the action).  Once the renewals have completed,
+               the completion handler will automatically issue a
+               commandline continuation message.  However, if there
+               were no identities to renew, then we have to issue the
+               message ourselves.
+            */
+
+            InterlockedIncrement(&khm_startup.pending_renewals);
+
+            khm_cred_renew_all_identities();
+
+            pending_renewals = InterlockedDecrement(&khm_startup.pending_renewals);
+
+            if (pending_renewals == 0 && khm_startup.processing)
+                kmq_post_message(KMSG_ACT, KMSG_ACT_CONTINUE_CMDLINE, 0, 0);
+        }
         break;
 
     case KMSG_CRED_REFRESH:
@@ -380,6 +555,11 @@ void khm_cred_set_default(void)
     khui_context_release(&ctx);
 }
 
+void khm_cred_set_default_identity(khm_handle identity)
+{
+    kcdb_identity_set_default(identity);
+}
+
 void khm_cred_destroy_creds(khm_boolean sync, khm_boolean quiet)
 {
     khui_action_context * pctx;
@@ -397,18 +577,18 @@ void khm_cred_destroy_creds(khm_boolean sync, khm_boolean quiet)
         wchar_t title[256];
         wchar_t message[256];
 
-        LoadString(khm_hInstance, 
-                   IDS_ALERT_NOSEL_TITLE, 
-                   title, 
+        LoadString(khm_hInstance,
+                   IDS_ALERT_NOSEL_TITLE,
+                   title,
                    ARRAYLENGTH(title));
 
-        LoadString(khm_hInstance, 
-                   IDS_ALERT_NOSEL, 
-                   message, 
+        LoadString(khm_hInstance,
+                   IDS_ALERT_NOSEL,
+                   message,
                    ARRAYLENGTH(message));
 
-        khui_alert_show_simple(title, 
-                               message, 
+        khui_alert_show_simple(title,
+                               message,
                                KHERR_WARNING);
 
         khui_context_release(pctx);
@@ -435,6 +615,106 @@ void khm_cred_destroy_creds(khm_boolean sync, khm_boolean quiet)
     _end_task();
 }
 
+void khm_cred_destroy_identity(khm_handle identity)
+{
+    khui_action_context * pctx;
+    wchar_t idname[KCDB_IDENT_MAXCCH_NAME];
+    khm_size cb;
+
+    if (identity == NULL)
+        return;
+
+    pctx = PMALLOC(sizeof(*pctx));
+#ifdef DEBUG
+    assert(pctx);
+#endif
+
+    khui_context_create(pctx,
+                        KHUI_SCOPE_IDENT,
+                        identity,
+                        KCDB_CREDTYPE_INVALID,
+                        NULL);
+
+    cb = sizeof(idname);
+    kcdb_identity_get_name(identity, idname, &cb);
+
+    _begin_task(KHERR_CF_TRANSITIVE);
+    _report_sr1(KHERR_NONE, IDS_CTX_DESTROY_ID, _dupstr(idname));
+    _describe();
+
+    kmq_post_message(KMSG_CRED,
+                     KMSG_CRED_DESTROY_CREDS,
+                     0,
+                     (void *) pctx);
+
+    _end_task();
+}
+
+void khm_cred_renew_all_identities(void)
+{
+    khm_size count;
+    khm_size cb = 0;
+    khm_size n_idents = 0;
+    khm_int32 rv;
+    wchar_t * ident_names = NULL;
+    wchar_t * this_ident;
+
+    kcdb_credset_get_size(NULL, &count);
+
+    /* if there are no credentials, we just skip over the renew
+       action. */
+
+    if (count == 0)
+        return;
+
+    ident_names = NULL;
+
+    while (TRUE) {
+        if (ident_names) {
+            PFREE(ident_names);
+            ident_names = NULL;
+        }
+
+        cb = 0;
+        rv = kcdb_identity_enum(KCDB_IDENT_FLAG_EMPTY, 0,
+                                NULL,
+                                &cb, &n_idents);
+
+        if (n_idents == 0 || rv != KHM_ERROR_TOO_LONG ||
+            cb == 0)
+            break;
+
+        ident_names = PMALLOC(cb);
+        ident_names[0] = L'\0';
+
+        rv = kcdb_identity_enum(KCDB_IDENT_FLAG_EMPTY, 0,
+                                ident_names,
+                                &cb, &n_idents);
+
+        if (KHM_SUCCEEDED(rv))
+            break;
+    }
+
+    if (ident_names) {
+        for (this_ident = ident_names;
+             this_ident && *this_ident;
+             this_ident = multi_string_next(this_ident)) {
+            khm_handle ident;
+
+            if (KHM_FAILED(kcdb_identity_create(this_ident, 0,
+                                                &ident)))
+                continue;
+
+            khm_cred_renew_identity(ident);
+
+            kcdb_identity_release(ident);
+        }
+
+        PFREE(ident_names);
+        ident_names = NULL;
+    }
+}
+
 void khm_cred_renew_identity(khm_handle identity)
 {
     khui_new_creds * c;
@@ -453,6 +733,12 @@ void khm_cred_renew_identity(khm_handle identity)
     _report_sr0(KHERR_NONE, IDS_CTX_RENEW_CREDS);
     _describe();
 
+    /* if we are calling this while processing startup actions, we
+       need to keep track of how many we have issued. */
+    if (khm_startup.processing) {
+        InterlockedIncrement(&khm_startup.pending_renewals);
+    }
+
     kmq_post_message(KMSG_CRED, KMSG_CRED_RENEW_CREDS, 0, (void *) c);
 
     _end_task();
@@ -476,6 +762,12 @@ void khm_cred_renew_cred(khm_handle cred)
     _report_sr0(KHERR_NONE, IDS_CTX_RENEW_CREDS);
     _describe();
 
+    /* if we are calling this while processing startup actions, we
+       need to keep track of how many we have issued. */
+    if (khm_startup.processing) {
+        InterlockedIncrement(&khm_startup.pending_renewals);
+    }
+
     kmq_post_message(KMSG_CRED, KMSG_CRED_RENEW_CREDS, 0, (void *) c);
 
     _end_task();
@@ -494,6 +786,12 @@ void khm_cred_renew_creds(void)
     _report_sr0(KHERR_NONE, IDS_CTX_RENEW_CREDS);
     _describe();
 
+    /* if we are calling this while processing startup actions, we
+       need to keep track of how many we have issued. */
+    if (khm_startup.processing) {
+        InterlockedIncrement(&khm_startup.pending_renewals);
+    }
+
     kmq_post_message(KMSG_CRED, KMSG_CRED_RENEW_CREDS, 0, (void *) c);
 
     _end_task();
@@ -510,6 +808,7 @@ void khm_cred_change_password(wchar_t * title)
 
     khui_cw_create_cred_blob(&nc);
     nc->subtype = KMSG_CRED_PASSWORD;
+    dialog_nc = nc;
 
     khui_context_get(&nc->ctx);
 
@@ -560,6 +859,31 @@ void khm_cred_change_password(wchar_t * title)
     }
 }
 
+void
+khm_cred_obtain_new_creds_for_ident(khm_handle ident, wchar_t * title)
+{
+    khui_action_context ctx;
+
+    if (ident == NULL)
+        khm_cred_obtain_new_creds(title);
+
+    khui_context_get(&ctx);
+
+    khui_context_set(KHUI_SCOPE_IDENT,
+                     ident,
+                     KCDB_CREDTYPE_INVALID,
+                     NULL,
+                     NULL,
+                     0,
+                     NULL);
+
+    khm_cred_obtain_new_creds(title);
+
+    khui_context_set_indirect(&ctx);
+
+    khui_context_release(&ctx);
+}
+
 void khm_cred_obtain_new_creds(wchar_t * title)
 {
     khui_new_creds * nc;
@@ -571,6 +895,7 @@ void khm_cred_obtain_new_creds(wchar_t * title)
 
     khui_cw_create_cred_blob(&nc);
     nc->subtype = KMSG_CRED_NEW_CREDS;
+    dialog_nc = nc;
 
     khui_context_get(&nc->ctx);
 
@@ -639,7 +964,7 @@ void khm_cred_obtain_new_creds(wchar_t * title)
         _report_sr0(KHERR_NONE, IDS_CTX_NEW_CREDS);
         _describe();
 
-        kmq_post_message(KMSG_CRED, KMSG_CRED_NEW_CREDS, 0, 
+        kmq_post_message(KMSG_CRED, KMSG_CRED_NEW_CREDS, 0,
                          (void *) nc);
 
         _end_task();
@@ -654,7 +979,7 @@ void khm_cred_obtain_new_creds(wchar_t * title)
 /* this is called by khm_cred_dispatch_process_message and the
    kmsg_cred_completion to initiate and continue checked broadcasts of
    KMSG_CRED_DIALOG_PROCESS messages.
-   
+
    Returns TRUE if more KMSG_CRED_DIALOG_PROCESS messages were
    posted. */
 BOOL khm_cred_dispatch_process_level(khui_new_creds *nc)
@@ -715,7 +1040,7 @@ BOOL khm_cred_dispatch_process_level(khui_new_creds *nc)
     return cont;
 }
 
-void 
+void
 khm_cred_dispatch_process_message(khui_new_creds *nc)
 {
     khm_size i;
@@ -760,7 +1085,7 @@ khm_cred_dispatch_process_message(khui_new_creds *nc)
             kcdb_identity_get_name(nc->ctx.identity, wsinsert, &cbsize);
         else if (nc->ctx.scope == KHUI_SCOPE_CREDTYPE) {
             if (nc->ctx.identity != NULL)
-                kcdb_identity_get_name(nc->ctx.identity, wsinsert, 
+                kcdb_identity_get_name(nc->ctx.identity, wsinsert,
                                        &cbsize);
             else
                 kcdb_credtype_get_name(nc->ctx.cred_type, wsinsert,
@@ -771,7 +1096,7 @@ khm_cred_dispatch_process_message(khui_new_creds *nc)
             StringCbCopy(wsinsert, sizeof(wsinsert), L"(?)");
         }
 
-        _report_sr1(KHERR_NONE, IDS_CTX_PROC_RENEW_CREDS, 
+        _report_sr1(KHERR_NONE, IDS_CTX_PROC_RENEW_CREDS,
                     _cstr(wsinsert));
         _resolve();
     } else if (nc->subtype == KMSG_CRED_PASSWORD) {
@@ -800,7 +1125,7 @@ khm_cred_dispatch_process_message(khui_new_creds *nc)
     if (nc->subtype == KMSG_CRED_RENEW_CREDS)
         kmq_post_message(KMSG_CRED, KMSG_CRED_END, 0, (void *) nc);
     else
-        PostMessage(nc->hwnd, KHUI_WM_NC_NOTIFY, 
+        PostMessage(nc->hwnd, KHUI_WM_NC_NOTIFY,
                     MAKEWPARAM(0, WMNC_DIALOG_PROCESS_COMPLETE), 0);
 }
 
@@ -825,16 +1150,8 @@ khm_cred_process_startup_actions(void) {
        rest of the startup actions. */
     do {
         if (khm_startup.init) {
-            if (defident)
-                khui_context_set(KHUI_SCOPE_IDENT,
-                                 defident,
-                                 KCDB_CREDTYPE_INVALID,
-                                 NULL, NULL, 0,
-                                 NULL);
-            else
-                khui_context_reset();
 
-            khm_cred_obtain_new_creds(NULL);
+            khm_cred_obtain_new_creds_for_ident(defident, NULL);
             khm_startup.init = FALSE;
             break;
         }
@@ -842,32 +1159,37 @@ khm_cred_process_startup_actions(void) {
         if (khm_startup.import) {
             khm_cred_import();
             khm_startup.import = FALSE;
+
+            /* we also set the renew command to false here because we
+               trigger a renewal for all the identities at the end of
+               the import operation anyway. */
+            khm_startup.renew = FALSE;
             break;
         }
 
         if (khm_startup.renew) {
-            khm_size count;
-
-            kcdb_credset_get_size(NULL, &count);
+            LONG pending_renewals;
 
             /* if there are no credentials, we just skip over the
                renew action. */
 
             khm_startup.renew = FALSE;
 
-            if (count != 0) {
-                if (defident)
-                    khui_context_set(KHUI_SCOPE_IDENT,
-                                     defident,
-                                     KCDB_CREDTYPE_INVALID,
-                                     NULL, NULL, 0,
-                                     NULL);
-                else
-                    khui_context_reset();
+            InterlockedIncrement(&khm_startup.pending_renewals);
 
-                khm_cred_renew_creds();
+            khm_cred_renew_all_identities();
+
+            pending_renewals = InterlockedDecrement(&khm_startup.pending_renewals);
+
+            if (pending_renewals != 0)
                 break;
-            }
+
+            /* if there were no pending renewals, then we just fall
+               through. This means that either there were no
+               identities to renew, or all the renewals completed.  If
+               all the renewals completed, then the commandline
+               contiuation message wasn't triggered.  Either way, we
+               must fall through if the count is zero. */
         }
 
         if (khm_startup.destroy) {
@@ -875,23 +1197,29 @@ khm_cred_process_startup_actions(void) {
             khm_startup.destroy = FALSE;
 
             if (defident) {
-                khui_context_set(KHUI_SCOPE_IDENT,
-                                 defident,
-                                 KCDB_CREDTYPE_INVALID,
-                                 NULL, NULL, 0,
-                                 NULL);
-
-                khm_cred_destroy_creds(FALSE, FALSE);
+                khm_cred_destroy_identity(defident);
                 break;
             }
         }
 
         if (khm_startup.autoinit) {
-            khm_size count;
+            khm_size count = 0;
+            khm_handle credset = NULL;
+            khm_int32 ctype_ident = KCDB_CREDTYPE_INVALID;
+            khm_int32 delta = 0;
 
             khm_startup.autoinit = FALSE;
 
-            kcdb_credset_get_size(NULL, &count);
+            kcdb_credset_create(&credset);
+            kcdb_identity_get_type(&ctype_ident);
+
+            kcdb_credset_collect(credset, NULL,
+                                 defident, ctype_ident,
+                                 &delta);
+
+            kcdb_credset_get_size(credset, &count);
+
+            kcdb_credset_delete(credset);
 
             if (count == 0) {
                 if (defident)
@@ -916,9 +1244,19 @@ khm_cred_process_startup_actions(void) {
             break;
         }
 
+        if (khm_startup.display & SOPTS_DISPLAY_HIDE) {
+            khm_hide_main_window();
+        } else if (khm_startup.display & SOPTS_DISPLAY_SHOW) {
+            khm_show_main_window();
+        }
+        khm_startup.display = 0;
+
         /* when we get here, then we are all done with the command
            line stuff */
         khm_startup.processing = FALSE;
+        khm_startup.remote = FALSE;
+
+        kmq_post_message(KMSG_ACT, KMSG_ACT_END_CMDLINE, 0, 0);
     } while(FALSE);
 
     if (defident)
@@ -932,7 +1270,9 @@ khm_cred_begin_startup_actions(void) {
     if (khm_startup.seen)
         return;
 
-    if (KHM_SUCCEEDED(khc_open_space(NULL, L"CredWindow", 0, &csp_cw))) {
+    if (!khm_startup.remote &&
+        KHM_SUCCEEDED(khc_open_space(NULL, L"CredWindow", 0, &csp_cw))) {
+
         khm_int32 t = 0;
 
         khc_read_int32(csp_cw, L"Autoinit", &t);
@@ -948,6 +1288,25 @@ khm_cred_begin_startup_actions(void) {
 
     }
 
+    /* if this is a remote request, and no specific options were
+       specified other than --renew, then we perform the default
+       action, as if the user clicked on the tray icon. */
+    if (khm_startup.remote &&
+        !khm_startup.exit &&
+        !khm_startup.destroy &&
+        !khm_startup.autoinit &&
+        !khm_startup.init &&
+        !khm_startup.remote_exit &&
+        !khm_startup.import &&
+        !khm_startup.display) {
+
+        khm_int32 def_action = khm_get_default_notifier_action();
+
+        if (def_action > 0) {
+            khui_action_trigger(def_action, NULL);
+        }
+    }
+
     khm_startup.seen = TRUE;
     khm_startup.processing = TRUE;
 
