@@ -26,6 +26,8 @@
 /* $Id$ */
 
 #define NOEXPORT
+#define _NIMLIB_
+
 #include<khuidefs.h>
 #include<intaction.h>
 #include<utils.h>
@@ -81,8 +83,9 @@ khui_action_ref khui_menu_toolbars[] = {
 
 khui_action_ref khui_menu_view[] = {
     MENU_ACTION(KHUI_ACTION_LAYOUT_MINI),
-    MENU_SUBMENU(KHUI_MENU_COLUMNS),
     MENU_SUBMENU(KHUI_MENU_LAYOUT),
+    MENU_ACTION(KHUI_ACTION_VIEW_ALL_IDS),
+    MENU_SUBMENU(KHUI_MENU_COLUMNS),
 #if 0
     /* not implemented yet */
     MENU_SUBMENU(KHUI_MENU_TOOLBARS),
@@ -159,6 +162,8 @@ khui_action_ref khui_menu_ico_ctx_min[] = {
     MENU_ACTION(KHUI_ACTION_IMPORT),
     MENU_SUBMENU(KHUI_MENU_DESTROY_CRED),
     MENU_SEP(),
+    MENU_SUBMENU(KHUI_MENU_SETDEF),
+    MENU_SEP(),
     MENU_ACTION(KHUI_ACTION_PASSWD_ID),
     MENU_SEP(),
     MENU_ACTION(KHUI_ACTION_HELP_CTX),
@@ -176,6 +181,8 @@ khui_action_ref khui_menu_ico_ctx_normal[] = {
     MENU_ACTION(KHUI_ACTION_IMPORT),
     MENU_SUBMENU(KHUI_MENU_DESTROY_CRED),
     MENU_SEP(),
+    MENU_SUBMENU(KHUI_MENU_SETDEF),
+    MENU_SEP(),
     MENU_ACTION(KHUI_ACTION_PASSWD_ID),
     MENU_SEP(),
     MENU_ACTION(KHUI_ACTION_HELP_CTX),
@@ -205,6 +212,10 @@ khui_action_ref khui_menu_renew_cred[] = {
     MENU_END()
 };
 
+khui_action_ref khui_menu_setdef[] = {
+    MENU_END()
+};
+
 khui_action_ref khui_pmenu_tok_sel[] = {
     MENU_ACTION(KHUI_ACTION_RENEW_CRED),
     MENU_ACTION(KHUI_ACTION_DESTROY_CRED),
@@ -230,6 +241,7 @@ khui_menu_def khui_all_menus[] = {
     CONSTMENU(KHUI_MENU_COLUMNS, KHUI_MENUSTATE_CONSTANT | KHUI_MENUSTATE_SYSTEM, khui_menu_columns),
     CONSTMENU(KHUI_MENU_RENEW_CRED, KHUI_MENUSTATE_CONSTANT | KHUI_MENUSTATE_SYSTEM, khui_menu_renew_cred),
     CONSTMENU(KHUI_MENU_DESTROY_CRED, KHUI_MENUSTATE_CONSTANT | KHUI_MENUSTATE_SYSTEM, khui_menu_destroy_cred),
+    CONSTMENU(KHUI_MENU_SETDEF, KHUI_MENUSTATE_CONSTANT | KHUI_MENUSTATE_SYSTEM, khui_menu_setdef),
 
     /* toolbars */
     CONSTMENU(KHUI_TOOLBAR_STANDARD, KHUI_MENUSTATE_CONSTANT | KHUI_MENUSTATE_SYSTEM, khui_toolbar_standard),
@@ -263,12 +275,12 @@ HWND khui_hwnd_main;			/* main window, for notifying
                                            dispatching messages to the
                                            application. */
 
-KHMEXP void KHMAPI 
+KHMEXP void KHMAPI
 khui_init_actions(void) {
     InitializeCriticalSection(&cs_actions);
 }
 
-KHMEXP void KHMAPI 
+KHMEXP void KHMAPI
 khui_exit_actions(void) {
     DeleteCriticalSection(&cs_actions);
 }
@@ -304,7 +316,8 @@ khui_action_create(const wchar_t * name,
         !caption ||
         FAILED(StringCchLength(caption, KHUI_MAXCCH_SHORT_DESC, &s)) ||
         (tooltip && FAILED(StringCchLength(tooltip, KHUI_MAXCCH_SHORT_DESC, &s))) ||
-        (type != KHUI_ACTIONTYPE_TRIGGER && type != KHUI_ACTIONTYPE_TOGGLE)) {
+        (type != KHUI_ACTIONTYPE_TRIGGER && type != KHUI_ACTIONTYPE_TOGGLE &&
+         type != KHUI_ACTIONTYPE_IDENTITY)) {
         return 0;
     }
 
@@ -428,7 +441,7 @@ khui_action_delete(khm_int32 action) {
 
 #define MENU_NC_ITEMS 8
 
-KHMEXP khui_menu_def * KHMAPI 
+KHMEXP khui_menu_def * KHMAPI
 khui_menu_create(khm_int32 action)
 {
     khui_menu_def * d;
@@ -495,10 +508,11 @@ khui_action_trigger(khm_int32 action, khui_action_context * ctx) {
 
     if (ctx) {
 	khui_context_set_indirect(&save);
+        khui_context_release(&save);
     }
 }
 
-KHMEXP khui_menu_def * KHMAPI 
+KHMEXP khui_menu_def * KHMAPI
 khui_menu_dup(khui_menu_def * src)
 {
     khui_menu_def * d;
@@ -516,9 +530,9 @@ khui_menu_dup(khui_menu_def * src)
 
     for (i=0; i<n; i++) {
         if (src->items[i].flags & KHUI_ACTIONREF_PACTION) {
-            khui_menu_insert_paction(d, -1, src->items[i].p_action, src->items[i].flags);
+            khui_menu_insert_paction(d, (khm_size) -1, src->items[i].p_action, src->items[i].flags);
         } else {
-            khui_menu_insert_action(d, -1, src->items[i].action, 0);
+            khui_menu_insert_action(d, (khm_size) -1, src->items[i].action, 0);
         }
     }
 
@@ -527,7 +541,7 @@ khui_menu_dup(khui_menu_def * src)
     return d;
 }
 
-KHMEXP void KHMAPI 
+KHMEXP void KHMAPI
 khui_menu_delete(khui_menu_def * d)
 {
     int i;
@@ -782,16 +796,18 @@ khui_find_menu(khm_int32 id) {
 
 KHMEXP khui_action * KHMAPI
 khui_find_action(khm_int32 id) {
-    khui_action * act;
+    khui_action * act = NULL;
     int i;
 
-    act = khui_actions;
-    for(i=0;i<khui_n_actions;i++) {
-        if(act[i].cmd == id)
-            return &act[i];
-    }
+    if (id < KHUI_USERACTION_BASE) {
+        act = khui_actions;
+        for(i=0;i<khui_n_actions;i++) {
+            if(act[i].cmd == id)
+                return &act[i];
+        }
 
-    act = NULL;
+        return NULL;
+    }
 
     EnterCriticalSection(&cs_actions);
     if (id >= KHUI_USERACTION_BASE &&
@@ -991,9 +1007,9 @@ khui_create_global_accel_table(void) {
 
     accels = PMALLOC(sizeof(ACCEL) * khui_n_accel_global);
     for(i=0;i<khui_n_accel_global;i++) {
-        accels[i].cmd = khui_accel_global[i].cmd;
-        accels[i].fVirt = khui_accel_global[i].mod;
-        accels[i].key = khui_accel_global[i].key;
+        accels[i].cmd = (WORD) khui_accel_global[i].cmd;
+        accels[i].fVirt = (BYTE) khui_accel_global[i].mod;
+        accels[i].key = (WORD) khui_accel_global[i].key;
     }
 
     ha = CreateAcceleratorTable(accels, khui_n_accel_global);
@@ -1003,9 +1019,9 @@ khui_create_global_accel_table(void) {
     return ha;
 }
 
-KHMEXP khm_boolean KHMAPI 
-khui_get_cmd_accel_string(khm_int32 cmd, 
-                          wchar_t * buf, 
+KHMEXP khm_boolean KHMAPI
+khui_get_cmd_accel_string(khm_int32 cmd,
+                          wchar_t * buf,
                           khm_size bufsiz) {
     int i;
     khui_accel_def * def;
@@ -1111,9 +1127,9 @@ khui_get_cmd_accel_string(khm_int32 cmd,
             break;
 
         default:
-            if((def->key >= '0' && 
-                def->key <= '9') || 
-               (def->key >= 'A' && 
+            if((def->key >= '0' &&
+                def->key <= '9') ||
+               (def->key >= 'A' &&
                 def->key <= 'Z')) {
                 ap = mbuf;
                 mbuf[0] = (wchar_t) def->key;
@@ -1132,7 +1148,7 @@ khui_get_cmd_accel_string(khm_int32 cmd,
     } else {
         wchar_t mbuf[2];
 
-        mbuf[0] = def->key;
+        mbuf[0] = (wchar_t) def->key;
         mbuf[1] = L'\0';
 
         if(FAILED(StringCbCat(buf, bufsiz, mbuf)))
@@ -1225,13 +1241,13 @@ khuiint_copy_context(khui_action_context * ctxdest,
             ctxdest->headers[i].attr_id = ctxsrc->headers[i].attr_id;
             ctxdest->headers[i].cb_data = ctxsrc->headers[i].cb_data;
             if (ctxsrc->headers[i].cb_data > 0) {
-                ctxdest->headers[i].data = 
+                ctxdest->headers[i].data =
                     BYTEOFFSET(ctxdest->int_buf,
                                ctxdest->int_cb_used);
                 memcpy(ctxdest->headers[i].data,
                        ctxsrc->headers[i].data,
                        ctxsrc->headers[i].cb_data);
-                ctxdest->int_cb_used += 
+                ctxdest->int_cb_used +=
                     UBOUND32(ctxsrc->headers[i].cb_data);
             } else {
                 ctxdest->headers[i].data = NULL;
@@ -1248,7 +1264,7 @@ khuiint_copy_context(khui_action_context * ctxdest,
 #endif
 
         kcdb_credset_flush(ctxdest->credset);
-        
+
         kcdb_credset_extract_filtered(ctxdest->credset,
                                       ctxsrc->credset,
                                       khuiint_filter_selected,
@@ -1282,7 +1298,7 @@ khuiint_copy_context(khui_action_context * ctxdest,
 #endif
 }
 
-static void 
+static void
 khuiint_context_init(khui_action_context * ctx) {
     ctx->magic = KHUI_ACTION_CONTEXT_MAGIC;
     ctx->scope = KHUI_SCOPE_NONE;
@@ -1303,8 +1319,8 @@ khuiint_context_init(khui_action_context * ctx) {
 khui_action_context khui_ctx = {
     KHUI_ACTION_CONTEXT_MAGIC,
     KHUI_SCOPE_NONE,
-    NULL, 
-    KCDB_CREDTYPE_INVALID, 
+    NULL,
+    KCDB_CREDTYPE_INVALID,
     NULL,
     {
         {KCDB_ATTR_INVALID,NULL,0},
@@ -1386,10 +1402,10 @@ khui_context_create(khui_action_context * ctx,
     khuiint_copy_context(ctx, &tctx);
 }
 
-KHMEXP void KHMAPI 
-khui_context_set(khui_scope scope, 
-                 khm_handle identity, 
-                 khm_int32 cred_type, 
+KHMEXP void KHMAPI
+khui_context_set(khui_scope scope,
+                 khm_handle identity,
+                 khm_int32 cred_type,
                  khm_handle cred,
                  khui_header *headers,
                  khm_size n_headers,
@@ -1406,10 +1422,10 @@ khui_context_set(khui_scope scope,
                         0);
 }
 
-KHMEXP void KHMAPI 
-khui_context_set_ex(khui_scope scope, 
-                    khm_handle identity, 
-                    khm_int32 cred_type, 
+KHMEXP void KHMAPI
+khui_context_set_ex(khui_scope scope,
+                    khm_handle identity,
+                    khm_int32 cred_type,
                     khm_handle cred,
                     khui_header *headers,
                     khm_size n_headers,
@@ -1466,7 +1482,7 @@ khui_context_set_indirect(khui_action_context * ctx)
     LeaveCriticalSection(&cs_actions);
 }
 
-KHMEXP void KHMAPI 
+KHMEXP void KHMAPI
 khui_context_refresh(void) {
     khm_int32 flags;
 
@@ -1499,7 +1515,7 @@ khui_context_refresh(void) {
     kmq_post_message(KMSG_ACT, KMSG_ACT_REFRESH, 0, 0);
 }
 
-KHMEXP void KHMAPI 
+KHMEXP void KHMAPI
 khui_context_get(khui_action_context * ctx)
 {
     EnterCriticalSection(&cs_actions);
@@ -1514,7 +1530,7 @@ khui_context_get(khui_action_context * ctx)
     LeaveCriticalSection(&cs_actions);
 }
 
-KHMEXP void KHMAPI 
+KHMEXP void KHMAPI
 khui_context_release(khui_action_context * ctx)
 {
 #ifdef DEBUG
@@ -1542,7 +1558,7 @@ khui_context_release(khui_action_context * ctx)
 #endif
 }
 
-KHMEXP void KHMAPI 
+KHMEXP void KHMAPI
 khui_context_reset(void)
 {
     EnterCriticalSection(&cs_actions);
